@@ -23,14 +23,15 @@ data Const = Num Int
    deriving (Eq, Ord)
 
 instance Show Const where
-    show (Num i) = show i
-    show (Ch ch) = show ch
-    show (Bo False) = "false"
+    show (Num x) = show x
+    show (Ch '\0') = "'\\0'"
+    show (Ch c) = show c
     show (Bo True) = "true"
-    show (Re d) = show d
+    show (Bo False) = "false"
+    show (Re v) = show v
     show (Str str) = show str
-    show (Exc str i) = "exception(" ++ show str ++ ", " ++ show i ++ ")"
-    show Empty = "()"
+    show (Exc str i) = "exception("++show str++","++show i++")"
+    show (Empty) = error "Can't show an empty constant"
 
 data Name = UN String
 	  | MN (String,Int)
@@ -87,7 +88,7 @@ instance Show PrimType where
   show RealNum = "Float"
   show StringType = "String"
   show Pointer = "Ptr"
-  show Exception = "%Exception"
+  show Exception = "Exception"
   show Void = "Void"
 
 data Type = Prim PrimType
@@ -314,7 +315,7 @@ data Expr n = Global n String Int
 	    | Seq (Expr n) (Expr n)
 	    | Apply (Expr n) [Expr n]
             | ConApply (Expr n) [Expr n]
-	    | Partial (Expr n) [Expr n] Int
+	    | Partial Bool (Expr n) [Expr n] Int -- bool True if from a lambda
 	    | Foreign Type n [(Expr n,Type)]
 	    | While (Expr n) (Expr n)
 	    | DoWhile (Expr n) (Expr n)
@@ -484,7 +485,7 @@ data RawDecl = FB (String,Int,Name,Type,[FOpt], Binder Raw) String
 	     | CInc String
              | FMN String
 	     | Imp String
-	     | SearchImport String -- First pass only, when chasing modules
+	     | SearchImport Name -- First pass only, when chasing modules
 	     | Link String
 	     | TSyn (String,Int,Name,[Type],Type, Bool) -- bool for export
 	     | DDecl String Int [DOpt] Name [Type] [ConDecl] String
@@ -539,6 +540,27 @@ instance MonadPlus Result where
     mplus (Success x) _ = (Success x)
     mplus (Failure _ _ _) y = y
 
+-- Show simple expressions for default arguments as strings
+
+showDefaultArg :: Raw -> String
+showDefaultArg (RVar _ _ n) = showuser n
+showDefaultArg (RQVar _ _ n) = "@" ++ showuser n
+showDefaultArg (RConst _ _ c) = show c
+showDefaultArg (RApply _ _ r as) = showDefaultArg r ++ "(" ++ showargs as ++ ")"
+     where showargs [] = ""
+	   showargs [x] = showDefaultArg x
+	   showargs (x:xs) = showDefaultArg x ++ "," ++ showargs xs
+showDefaultArg (RArrayInit _ _ rs) = "[" ++ showargs rs ++ "]"
+     where showargs [] = ""
+	   showargs [x] = showDefaultArg x
+	   showargs (x:xs) = showDefaultArg x ++ "," ++ showargs xs
+showDefaultArg (RIndex _ _ r i) = showDefaultArg r ++ "["++showDefaultArg i++"]"
+showDefaultArg (RField _ _ n f) = showDefaultArg n ++ "." ++ showuser f
+showDefaultArg (RUnary _ _ op r) = showunop op ++ showDefaultArg r
+   where showunop Not = "!"
+	 showunop Neg = "-"
+-- TODO: The rest
+showDefaultArg _ = error "Invalid default argument, this probably shouldn't have parsed"
 -- Substitute a term for a variable in a raw term
 
 rawSubst :: Name -> Raw -> Raw -> Raw
