@@ -9,10 +9,12 @@
 module Errors where
 
 import AbsSyntax
+import Portability
 
 -- Represent errors as a datatype, so that they can be changed easily
 -- TODO: Not used yet, but it would be nice if it was!
 
+{-
 data Error = CantUnify Type Type
            | NotCasable Type
            | NotPrintable Type
@@ -38,3 +40,22 @@ data Error = CantUnify Type Type
            | GenericTypeError
            | InfiniteType
            | LessGeneral Type Type
+-}
+
+reportError :: String -> IO ()
+reportError err = do mcols <- environment "COLUMNS"
+                     let cols = case mcols of
+                                   Nothing -> 80
+                                   Just x -> read x :: Int
+                     putStrLn (format cols err)
+
+format :: Int -> String -> String
+format cols str = format' cols cols (words str)
+
+format' :: Int -> Int -> [String] -> String
+format' _ _ [] = ""
+format' cols i (x:xs) 
+    | (length x + 1) < i || ((length x + 1) >= cols && i == cols)
+        = x ++ (if null xs then "" else
+                " " ++ format' cols (i - (length x + 1)) xs)
+    | otherwise = "\n" ++ format' cols cols (x:xs)
