@@ -65,12 +65,19 @@ insertdecl mod excs globs ctxt t =
        gu mtch ds lseq ++ gu mtch ds rseq
    gu mtch ds (RReturn _ _ (RCase str l v exps)) 
        = {-gu mtch ds v ++ -} (concat (map (gua mtch ds) exps))
+   -- if we're returning something complicated (like a for loop generated
+   -- from a list comprehension, we'd better look for variables assigned
+   -- inside it.
+   gu mtch ds (RReturn _ _ a@(RSeq _ _ _ _)) = gu mtch ds a
    gu mtch ds (RReturn str l a) = []
    gu mtch ds (RApply str l fn args) =
        -- Names in applications must be declared or used already, except
-       -- in match patterns
+       -- in match patterns, and except if the arguments are closures.
+       -- (i.e. it's an argument which may cause assignment to happen)
        if mtch then (concat (map (gu True ds) args))
-          else []
+          else (concat (map (gcu ds) args))
+      where gcu ds rc@(RClosure _ _ _ _) = gu mtch ds rc
+            gcu _ _ = []
    gu mtch ds (RPartial str l fn args) = []
        -- Names in applications must be declared or used already.
        --checkdecl mtch fn ds (concat (map (gu mtch ds) args))
@@ -187,7 +194,9 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
       (exp',eqn,_) <- ti [] 0 t ideq vtype mkrexp
       return (exp',eqn)
  where 
-   mkrexp = getrtype (appsyn tys ftype)
+   mkrexp = case ftype of 
+                (Fn _ _ UnknownType) -> TyVar (MN ("rettype", 0))
+                _ -> getrtype (appsyn tys ftype)
    getrtype (Fn _ _ ret) = ret
 
    tiVar env next (RVar file line n) eqns exp disambig rexp =
@@ -277,7 +286,7 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
 	      return ((x:xs'),next')
    ti env next (RBind file line n t v sc) eqns exp rexp = do
       let syntin = appsyn tys t
-      synt <- normalise file line mod tys syntin
+      synt <- normalise False file line mod tys syntin
       let newenv = env ++ [(n,synt)]
       (vinf,veqns,next') <- ti env (next+1) v eqns synt rexp 
       (scinf,sceqns,next'') <- ti newenv next' sc veqns exp rexp
@@ -378,7 +387,7 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
       eqns' <- addEq file line (reqtype,exp, "partial application" ++ showraw f) feqns
       eqns'' <- addEq file line (newtv,ftype, "partial application" ++ showraw f) eqns'
 --      (finf,feqns,next'') <- ti env next' (RQVar file line f) aeqns reqtype
-      return (Partial finf achecks numleft, eqns'', next'')
+      return (Partial False finf achecks numleft, eqns'', next'')
      where mkatypes :: [Raw] -> Int -> ([Type],Int)
            mkatypes [] next = ([],next)
            mkatypes (x:xs) next = 
@@ -449,15 +458,41 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
 	   let counteri = length env
 	   let counterj = (length env)+1
 	   return (For counteri (Just counter1) counterj lcheck vinf vl, neweqns, next''')
-{-         t@(User (NS _ (UN "LazyArray")) _) -> do
-           -- reset(vals); while(!end(vals)) { l=getNext(vals); loop }
-	   let realcode = -- Ooh, lispy ;)
-	           bind cname t vals
-		   (seq (apply "reset" [count])
-		       (while (not (apply "end" [count]))
-			  (seq (assign l (apply "getNext" [count]))
-			       loop)))
-	   ti env (next+1) realcode eqns exp rexp-}
+-- convert into a call to 
+-- try {
+--    traverse(\(lt l, Int idxname) { loop; return true; }, vals);
+-- } catch(Return(v)) {
+--     return v;
+-- } (or equivalent VoidReturn if we're in a void function)
+-- inside loop, top level break becomes 'return false'
+-- VoidReturn becomes throw(VoidReturn), return x becomes throw(Return(x))
+-- Check it like it's an ordinary loop, then mangle the contents.
+         TyApp _ [argt] -> 
+             do let countername = case idxname of
+                                    (Just n) -> n
+                                    Nothing -> (MN ("counter", next'))
+            -- TMP HACK: don't care about the answer, just want to verify it
+                -- It's needlessly slow to do this twice. But while we get
+                -- it working it's much easier than manipulating typechecked
+                -- things where we have to deal with the new exception variable
+                -- we're about to introduce...
+	        (_,_,_) <- ti env' next'' loop veqns (Prim Void) rexp
+                lname <- case l of
+                           (RAName f l n) -> return n
+                           _ -> fail $ file ++ ":" ++ show line ++ ":for loop must assign to a variable"
+                let blocklam = RClosure file line [(lname, argt),
+                                                   (countername, Prim Number)]
+                                                  (RSeq file line modloop
+                                                        (RReturn file line 
+                                                           (RConst file line
+                                                             (Bo True))))
+                let code = RDeclare file line ((MN ("c", next'+1)), True)
+                                              UnknownType
+                           (RNewTryCatch file line 
+                           (RApply file line (RQVar file line (UN "traverse")) 
+                                            [blocklam, vals])
+                            (catchBlock rexp (next'+1)))
+                ti env (next'+2) code veqns exp rexp
          t -> fail $ file ++ ":" ++ show line ++ ":for loop cannot iterate over " ++ show t
       where seq x y = RSeq file line x y
 	    bind n t v b = RBind file line n t v b
@@ -469,6 +504,17 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
 	    not b = RUnary file line Not b
 	    assign l v = RAssign file line l v
 	    assignname l v = RAssign file line (RAName file line l) v
+            modloop = mkTraverse loop
+            catchBlock (Prim Void) next
+                = [RCatch file line (Left ((UN "Loop_VoidReturn"),[]))
+                          (RVoidReturn file line)]
+            catchBlock _ next
+                = [RCatch file line (Left ((UN "Loop_Return"), 
+                                           [(MN ("c", next))]))
+                         (RReturn file line 
+                            (RApply file line (RVar file line (UN "subvert"))
+                              [(RQVar file line (MN ("c", next)))]))]
+
    ti env next (RTryCatch file line tr ca x fin) eqns exp rexp = do
       (trinf, treqns, next') <- ti env next tr eqns (Prim Void) rexp
       (cainf, caeqns, next'') <- ti env next' ca treqns (Prim Void) rexp
@@ -564,7 +610,7 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
        phi <- mkSubst veqns
        let vtype = subst phi newtv
        let (fudgedt, next'') = fudgevars vtype next'
-       lookt <- normalise file line mod tys fudgedt
+       lookt <- normalise False file line mod tys fudgedt
        (fty, arg, tag) <- getFieldType flds f lookt file line
        eqns' <- addEq file line (fty, exp, "field access '" ++ showuser f ++ "'") veqns
        return (Annotation (Line file line) (Field vinf f arg tag), 
@@ -702,7 +748,7 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
        phi <- mkSubst reqns
        let realrt = subst phi rt
        let (fudgedt, next'') = fudgevars realrt next'
-       lookt <- normalise file line mod tys fudgedt
+       lookt <- normalise False file line mod tys fudgedt
        (fty, arg, tag) <- getFieldType flds f lookt file line
        return (AField rcheck f arg tag, (appsyn tys fty), reqns, next'')
 
@@ -790,6 +836,7 @@ appsyn tys t = {- trace ((show t) ++ " to " ++ show (syn t)) $ -} syn t where
     syn (Array t) = Array (syn t)
     syn (User n) = User (resolve tys n)
     syn (TyApp n ts) = TyApp (syn n) (map syn ts)
+    syn UnknownType = UnknownType
 {-       where resolve [] n = n
 	     resolve ((cn@(NS _ bn),ti):xs) n | bn == n = cn
 	     resolve ((x,ti):xs) n | x == n = x
@@ -801,7 +848,7 @@ appsyn tys t = {- trace ((show t) ++ " to " ++ show (syn t)) $ -} syn t where
     syn (TyVar n) = TyVar n
 
 synctx tys (n,ty) = (n,appsyn tys ty)
-normalisectx f l mod tys (n,ty) = do ty' <- normalise f l mod tys ty
+normalisectx f l mod tys (n,ty) = do ty' <- normalise True f l mod tys ty
                                      return (n,ty')
 
 checkTy :: Monad m => Types -> Type -> Type -> String -> m ()
@@ -888,10 +935,10 @@ substTerm mod ctxt phi t = st t
      st (Apply e1 args) = do e1' <- st e1
 			     args' <- mapM st args
 			     return $ Apply e1' args'
-     st (Partial e1 args i) 
+     st (Partial b e1 args i) 
 	   = do e1' <- st e1
 		args' <- mapM st args
-		return $ Partial e1' args' i
+		return $ Partial b e1' args' i
      st (Foreign ty n args) = do args' <- fsubst args
 				 return $ Foreign (subst phi ty) n args'
          where fsubst [] = return []
@@ -1139,17 +1186,45 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((Link str):xs) copts =
 
 -- Global variables
 inferAll mod ctxt errs globs excs tags syns flds dump ((GlobDecl file line (n,t,v)):xs) copts =
-  do st <- normalise file line mod syns t
+  do st <- normalise False file line mod syns t
      let gid = length globs
      (rest, errs, ctxt', globs', excs', tags',syns') <- 
 	 inferAll mod ctxt errs ((n,(st,gid)):globs) excs tags syns flds dump xs copts
      vval <- checkGlob file line mod ctxt' globs excs tags syns flds v st copts
      return ((Glob (n,st,gid,vval):rest), errs, ctxt', ((n,(st,gid)):globs), excs', tags', syns')
 
+-- Constant functions with no type label. We need to do these separately,
+-- because we need to determine their type before typechecking other functions
+-- (usually we know the type of everything in advance)
+inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,Fn [] [] UnknownType,fopts,Defined xr) comm):xs) copts = 
+  do let xft' = Fn [] [] (TyVar (MN ("ret",0)))
+     let xrdec = insertdecl mod excs globs (addName ctxt x xft' fopts) xr
+     case infer mod ctxt globs excs tags syns flds xrdec 
+                xft' copts of
+        Failure err f l -> do
+              -- check the rest anyway to see if we can get more errors
+             (rest, errs', ctxt', globs', excs', tags',syns') <- 
+	         inferAll mod ctxt errs globs excs tags 
+                          syns flds dump xs copts
+             return (rest, err:errs', ctxt, globs, excs, tags, syns)
+        Success (xrv,xeq) -> do
+             xeq' <- return $! 
+                      (if dump then (trace (showeqns x xft' xeq) xeq) else xeq)
+	     phi' <- {- trace (show xrv) $ -} mkSubst xeq'
+	     xfn <- substTerm mod ctxt phi' xrv
+	     let xinft = subst phi' xft'
+             (rest, errs', ctxt', globs', excs', tags',syns') <- 
+	         inferAll mod (addName ctxt x xinft fopts) errs globs excs tags 
+                          syns flds dump xs copts
+ 	     let xannot = Annotation (FnBody (showuser x) file line) xfn
+             return ((FunBind (file,line,x,xinft,fopts,Defined xannot) comm xinft):rest,
+                     errs',
+		    (addName ctxt' x xinft fopts), globs', excs', tags', syns')
+
 -- Function definitions
 inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts,Defined xr) comm):xs) copts = 
   do let xftin = appsyn syns xt
-     xft <- normalise file line mod syns xftin
+     xft <- normalise False file line mod syns xftin
 --     trace (showuser x++":"++show xft) $ return ()
      -- If it's been defined before, and it's a DefaultDef, just ignore it.
      if (DefaultDef `elem` fopts) && (defined ctxt x)
@@ -1175,8 +1250,13 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts
 	        xfn <- substTerm mod ctxt phi' xrv
 	        let xinft = subst phi' xft'
 	        checkEq file line xinft xft'
-	        checkReturn file line (showuser x) xft' xfn
+	        checkReturn file line (showuser x) xinft xfn
 	        let xannot = Annotation (FnBody (showuser x) file line) xfn
+                -- We ought to return xinft as the type, since that's what
+                -- we inferred. Alas, we currently need to use xft' instead
+                -- since we already put it in the context when we checked the
+                -- rest of the module. Need to rethink this whole process 
+                -- really...
 	        return {-$ trace ((show xrv)++"\n"++(show xfn)) $ -}
                    ((FunBind (file,line,x,xft',fopts,Defined xannot) comm xt):rest,
                      errs',
@@ -1198,7 +1278,7 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts
 -- Imported function declaration
 inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts,Unbound) comm):xs) copts =
   do let xftin = appsyn syns (mkfuntype xt)
-     xft <- normalise file line mod syns xftin
+     xft <- normalise False file line mod syns xftin
 --     let xft = trace (show xt)mkfuntype xt
      let (newctxt,newopts) = mergeIntoCtxt x xt fopts ctxt
      (rest, errs, ctxt', globs', excs', tags', syns') <- inferAll mod (addName newctxt x xft newopts) errs globs excs tags syns flds dump xs copts
@@ -1210,7 +1290,7 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts
 -- Imported and inlinable function declaration
 inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts, ExtInlinable xr) comm):xs) copts =
   do let xftin = appsyn syns (mkfuntype xt)
-     xft <- normalise file line mod syns xftin
+     xft <- normalise False file line mod syns xftin
 --     let xft = trace (show xt)mkfuntype xt
      let (newctxt,newopts) = mergeIntoCtxt x xt fopts ctxt
      (rest, errs, ctxt', globs', excs', tags', syns') <- inferAll mod (addName newctxt x xft newopts) errs globs excs tags syns flds dump xs copts
@@ -1240,7 +1320,7 @@ inferAll mod ctxt errs globs excs tags tydefs flds dump ((TSyn (file,line,n,ps,t
      (rest, errs, ctxt', globs', excs', tags',syns') <- 
 	 inferAll mod ctxt errs globs excs tags newdefs flds dump xs copts
      -- normalise to check for cycles
-     foo <- normalise file line mod syns' (TyApp (User n) ps)
+     foo <- normalise False file line mod syns' (TyApp (User n) ps)
      return $ ((TySyn (file,line,n,(map getname ps),ty,exp):rest), errs, 
                ctxt', globs', excs', tags', syns')
   where getname (TyVar x) = x
@@ -1281,7 +1361,7 @@ inferAll mod ctxt errs globs excs tags tydefs flds dump ((DDecl f l dopts n tys
                               ncon <- normcon x n
                               return $ ncon:rest
        normcon (Con n ty ns b) newdefs = 
-           do normty <- normalise f l mod newdefs (appsyn newdefs ty)
+           do normty <- normalise False f l mod newdefs (appsyn newdefs ty)
               return $ Con n normty ns b
 
 -- Exception declarations
@@ -1295,7 +1375,7 @@ inferAll mod ctxt errs globs excs tags tydefs flds dump ((ExtExc f l n tys):xs)
 	 inferAll mod exctxt errs globs newexcs tags tydefs flds dump xs copts
      return $ (exfnbody:rest, errs, ctxt', globs', excs', tags', syns')
 inferAll mod ctxt errs globs excs tags tydefs flds dump ((ExcDecl f l n tysin comm):xs) copts =
-  do tys <- mapM (normalise f l mod tydefs) tysin
+  do tys <- mapM (normalise False f l mod tydefs) tysin
      let fulln = NS mod n
      let newexcs = addToCtxt excs fulln tys
      -- add a function of called n of type Exception(tys), which constructs
@@ -1360,7 +1440,7 @@ getFuns ctxt f l mod ((Con n ty an codegen):xs) syns i
        if codegen 
 	  then checkRpt f l n ty ctxt False -- Check the name isn't used elsewhere
 	  else return () -- Do nothing if it's from somewhere else
-       normty <- normalise f l mod syns (appsyn syns ty)
+       normty <- normalise False f l mod syns (appsyn syns ty)
      --  trace (showuser n ++ ":" ++ show normty) $ return ()
        return (((FunBind (f,l,n,normty,[Inline,Pure,Public,Constructor],DataCon i (getarglen ty) codegen) "" normty):rest),
 	       (getFields (appsyn syns ty) an 0 i) ++ flds, codegen)
@@ -1430,7 +1510,7 @@ checkData f l syns tys (c:cs) = do checkConDecl f l syns tys c
 				   checkData f l syns tys cs
 				   
 checkConDecl f l syns tys (Con n ty _ _) = 
-    do --normty <- normalise f l syns ty
+    do --normty <- normalise False f l syns ty
        allDeclared f l n syns tys ty
 
 allDeclared f l n dtys tys ty = {- trace (show ty) $ -} ad ty
@@ -1452,7 +1532,7 @@ checkUserData f l mod syns (c:cs) = do checkUserDecl f l mod syns c
 				       checkUserData f l mod syns cs
 				   
 checkUserDecl f l mod syns (Con n ty _ _) 
-  = do normty <- normalise f l mod syns ty
+  = do normty <- normalise False f l mod syns ty
        userDeclared f l n syns normty
 
 userDeclared f l n dtys ty = ad ty
