@@ -253,7 +253,11 @@ getUserVars = nub.gv
     where gv (Fn ns ts t) = concat (map gv (t:ts))
 	  gv (Array t) = gv t
 	  gv (TyApp n ts) = concat (map gv (n:ts))
-	  gv (TyVar (UN n)) = [UN n]
+-- Actually it needs to be every variable so that constants get their
+-- types inferred and generalised correctly - it was a faulty assumption
+-- that all global names would have user defined type variables!
+--	  gv (TyVar (UN n)) = [UN n]
+	  gv (TyVar x) = [x]
 	  gv _ = []
 
 lvaltoexp :: RAssign -> Raw
@@ -261,16 +265,6 @@ lvaltoexp (RAName f l n) = RVar f l n
 lvaltoexp (RAIndex f l lv r) = RIndex f l (lvaltoexp lv) r
 lvaltoexp (RAField f l lv r) = RField f l (lvaltoexp lv) r
 
-showconst (Num x) = show x
-showconst (Ch '\0') = "'\\0'"
-showconst (Ch c) = show c
-showconst (Bo True) = "true"
-showconst (Bo False) = "false"
-showconst (Str str) = show str
-showconst (Exc str i) = "exception("++show str++","++show i++")"
-showconst (Empty) = error "Can't show an empty constant"
-
-
 -- Lookup in the type context (ignores namespace if no ambiguity, returns
 -- fully qualified type name)
 typelookup :: Name -> -- Current module
@@ -305,8 +299,10 @@ exceptlookup mod e ei = returnName (nubnames (lookupname mod e ei))
 tyapp u [] = u
 tyapp u ts = TyApp u ts
 
-normalise :: Monad m => String -> Int -> Name -> Types -> Type -> m Type
-normalise f l mod ti t = tn True [] t
+normalise :: Monad m => 
+             Bool -> -- Unknown types should cause an error
+             String -> Int -> Name -> Types -> Type -> m Type
+normalise unk f l mod ti t = tn True [] t
  where
    tn top u (Fn ds ts t) 
        = do ts' <- mapM (tn top u) ts
@@ -317,7 +313,9 @@ normalise f l mod ti t = tn True [] t
    tn top u t@(User n) = 
        case typelookup mod n ti of
           un@(UnknownName _) -> 
-               return $ User (fixup mod n)
+               if unk 
+                  then fail $ f ++ ":" ++ show l ++ ":" ++ show un
+                  else return $ User (fixup mod n)
           am@(Ambiguous n xs) -> fail $ f ++ ":" ++ show l ++ ":" ++ show am
 	  (Got (fqn, x)) -> applyTI top u fqn [] x
    tn top u (TyApp (User n) []) = tn top u (User n)
@@ -483,7 +481,7 @@ mapsubexpr f mf expr = app expr
 	app (Seq a b) = Seq (f a) (f b)
 	app (Apply fn as) = Apply (f fn) (applys as)
 	app (ConApply fn as) = ConApply (f fn) (applys as)
-	app (Partial fn as i) = Partial (f fn) (applys as) i
+	app (Partial b fn as i) = Partial b (f fn) (applys as) i
 	app (Foreign ty n es) = Foreign ty n 
 			        (zip (applys (map fst es)) (map snd es))
 	app (While e b) = While (f e) (f b)
@@ -542,7 +540,7 @@ foldsubexpr f com def expr = app expr
 	app (Seq a b) = (f a) `com` (f b)
 	app (Apply fn as) = (f fn) `com` (applys as)
 	app (ConApply fn as) = (f fn) `com` (applys as)
-	app (Partial fn as i) = (f fn) `com` (applys as)
+	app (Partial b fn as i) = (f fn) `com` (applys as)
 	app (Foreign ty n es) = applys (map fst es)
 	app (While e b) = (f e) `com` (f b)
 	app (DoWhile e b) = (f e) `com` (f b)
@@ -597,7 +595,7 @@ locsUsed (AssignApp a e) = alocsUsed a ++ locsUsed e
 locsUsed (Seq x y) = locsUsed x ++ locsUsed y
 locsUsed (Apply f as) = locsUsed f ++ concat (map locsUsed as)
 locsUsed (ConApply f as) = locsUsed f ++ concat (map locsUsed as)
-locsUsed (Partial f as _) = locsUsed f ++ concat (map locsUsed as)
+locsUsed (Partial b f as _) = locsUsed f ++ concat (map locsUsed as)
 locsUsed (Foreign _ _  as) = concat (map locsUsed (map fst as))
 locsUsed (While x y) = locsUsed x ++ locsUsed y
 locsUsed (DoWhile x y) = locsUsed x ++ locsUsed y
@@ -670,7 +668,7 @@ modVar (NewTryCatch e cs) = modVar e ++ concat (map cmodVar cs)
 modVar (Apply _ args) = concat (map locsUsed args)
 modVar (ConApply _ args) = concat (map locsUsed args)
 -- May be over conservative, but unlikely to happen much in practice.
-modVar (Partial _ args _) = concat (map locsUsed args)
+modVar (Partial _ _ args _) = concat (map locsUsed args)
 modVar (Foreign _ _ args) = concat (map locsUsed (map fst args))
 modVar (Case e alts) = modVar e ++ concat (map inAlt alts)
 modVar (If x t e) = modVar x ++ modVar t ++ modVar e
