@@ -267,7 +267,7 @@ lexer cont ('#':'!':cs) = cont TokenShebang (stripToNL cs)
 	 stripToNL (x:cs) = stripToNL cs
 lexer cont (c:cs) = lexError c cs
 
-lexError c s l = failP (show l ++ ": Unrecognised token '" ++ [c] ++ "'\n") s l
+lexError c cs fn ld mod l = failP (fn ++ ":" ++ show l ++ ":Unrecognised token '" ++ [c] ++ "'\n") cs fn ld mod l
 
 lexerEatComment nls cont ('*':'/':cs) 
     = \fn ld mod line v -> lexer cont cs fn ld mod (line+nls) v
@@ -413,10 +413,10 @@ lexSpecial cont cs =
       ("VM",rest) -> cont TokenVM rest
       ("length",rest) -> cont TokenLength rest
       ("startup",rest) -> cont TokenStartup rest
-      ("Exception",rest) -> cont TokenException rest
+--      ("Exception",rest) -> cont TokenException rest
       ("except",rest) -> cont TokenExtException rest
       ("try",rest) -> cont TokenTry rest
-      ("case",rest) -> cont TokenMatch rest
+--      ("case",rest) -> cont TokenMatch rest
       ("of",rest) -> cont TokenWith rest
       (thing,rest) -> cont TokenMod (thing++rest)
 
