@@ -218,7 +218,7 @@ getmatches (Apply x xs) (Apply y ys) = do m <- getmatches x y
 getmatches (ConApply x xs) (ConApply y ys) = do m <- getmatches x y
 					        ms <- getmatcheses xs ys
 					        return $ m ++ ms
-getmatches (Partial x xs i) (Partial y ys j) | i == j =
+getmatches (Partial a x xs i) (Partial b y ys j) | i == j && a == b =
 					    do m <- getmatches x y
 					       ms <- getmatcheses xs ys
 					       return $ m ++ ms
