@@ -41,6 +41,13 @@ dropUnused atys used exp = du (mapsubexpr (dropUnused atys used) Metavar exp)
                                 | otherwise = Noop
      du x = x
 
+
+-- Forget about any variables modified in e 
+forgetFrom :: Expr Name -> State ([Prop], [ArgType], Bool) ()
+forgetFrom e = do (ps, as, act) <- get
+                  put (dropMod (modified e) ps, as, act)
+
+
 doProp :: Expr Name -> State ([Prop], [ArgType], Bool) (Expr Name)
 -- Do it!
 doProp (Loc i) = do (ps, as, action) <- get
@@ -115,13 +122,11 @@ doProp (If x t e) = do x' <- doProp x
                        t' <- doProp t
                        -- Anything modified in then branch should be forgotten
                        -- for the else branch
-                       (ps, as, act) <- get
-                       put (dropMod (modified t) ps, as, act)
+                       forgetFrom t
                        e' <- doProp e
                        -- Anything modified in the branches has not necessarily
                        -- been executed, so forget those assignments
-                       (ps, as, act) <- get
-                       put (dropMod (modified e) ps, as, act)
+                       forgetFrom e
                        return (If x' t' e')
 doProp c@(Case e alts) = do let mods = modified c
                             e' <- doProp e
@@ -209,21 +214,27 @@ doProp (Apply f as) = do f' <- doProp f
    -- be a var arg, so don't propagate it.
                          as' <- mapM doPropArg as
                          return (Apply f' as')
-doProp (Partial f as i) = do f' <- doProp f
+doProp (Partial b f as i) = do f' <- doProp f
    -- On its own as a function argument, it gets boxed anyway, and it might
    -- be a var arg, so don't propagate it.
-                             as' <- mapM doPropArg as
-                             return (Partial f' as' i)
+                               as' <- mapM doPropArg as
+                               return (Partial b f' as' i)
 doProp (Foreign t n es) = do es' <- mapM doPropArg (map fst es)
                              return (Foreign t n (zip es' (map snd es)))
 doProp (NewTryCatch e cs) = do e' <- doProp e
+                               -- Forget what was modified for the catches
+                               forgetFrom e
                                cs' <- mapM doPropC cs
                                return (NewTryCatch e' cs')
-   where doPropC (Catch (Left (n,es)) e) = do es' <- mapM doPropArg es
-                                              e' <- doProp e
-                                              return (Catch (Left (n,es')) e')
+   where doPropC (Catch (Left (n,es)) e) 
+             = do es' <- mapM doPropArg es
+                  e' <- doProp e
+                  -- Forget what we modified
+                  forgetFrom e
+                  return (Catch (Left (n,es')) e')
          doPropC (Catch (Right r) e) = do r' <- doProp r
                                           e' <- doProp e
+                                          forgetFrom e
                                           return (Catch (Right r') e')
 doProp (NewExcept es) = do es' <- mapM doPropArg es
                            return (NewExcept es')
