@@ -16,6 +16,9 @@ module TAC where
 import Language
 import Debug.Trace
 import List
+import Control.Monad.State
+import Data.Maybe
+
 import KeyGen
 import Dataflow
 
@@ -46,7 +49,7 @@ data TAC = DECLARE Var
 	 | SUBTOP
 	 | MULTOP
 	 | DIVTOP
-	 | MKARRAY Int
+	 | MKARRAY Var Int
 	 | USETMP Var -- Flag that Var was used, do nothing
 	 | DEREF
 	 | TMPSET Tmp Int
@@ -58,9 +61,9 @@ data TAC = DECLARE Var
 	 | TAILCALL Var
 	 | TAILCALLNAME Mangled
 	 | TAILCALLTOP
-	 | CLOSURE Mangled Int Int
-	 | CLOSURELOC Var Int 
-	 | CLOSURETOP Int 
+	 | CLOSURE Bool Mangled Int Int
+	 | CLOSURELOC Bool Var Int 
+	 | CLOSURETOP Bool Int 
 	 | FOREIGNCALL String String Type [Type] -- name, library, type, args
 	 | MKCON Int Int -- Create a constructor, tag and arity
 	 | MKCONZERO Int -- Create a constructor, tag with arity zero
@@ -207,7 +210,7 @@ data CompData = CD {
 		    alldecls :: Program,
                     varargs :: [Var],
                     -- All we've learned from dataflow analysis
-                    varInfo :: [Int] -- just unused variables currently
+                    varInfo :: [Int] -- We don't currently use this. It may yet be used for storing variables which don't overlap in the same place.
 		   }
 
 data Code = Code {
@@ -242,12 +245,11 @@ fcompile ds (Annotation a e) pure mod =
       return (Code pop args acode)
 fcompile ds (Lambda ivs args sc) pure mod 
     = do let vs' = getvarargs 0 (zip ivs (map fst args))
-         let (alllocs, vinfo) = findInfo sc
-         let unused = neverUsed alllocs vinfo
+         let (alllocs, vinfo) = findInfo ivs sc
          (code,lab',stmp') <- fcompile' sc 
                         (CD mod 
                             (map (\x -> (fst x, False)) args)
-                            0 Nothing 0 (updatecache cache (length args)) ds vs' unused) 
+                            0 Nothing 0 (updatecache cache (length args)) ds vs' []) 
                           0 0
 	 let acode = acompile ivs args
 	 let cachecode = case cache of
@@ -450,9 +452,9 @@ fcompile' (Apply f as) cd lab stmp
     where mkRcode lab stmp = return ([USETMP newtmp,PUSH (VAR newtmp)],lab,(stmp+1)) -- make space for return value
           newtmp = ((MN ("stmp",stmp)),True)
 
-fcompile' (Partial f as i) cd lab stmp
-    = do (pcode, lab', stmp') <- pushargs as cd lab stmp
-	 (ccode, lab'', stmp'') <- closure f (length as) i cd lab' stmp'
+fcompile' (Partial b f as i) cd lab stmp
+    = do (pcode, lab', stmp') <- pushargsHeap as cd lab stmp
+	 (ccode, lab'', stmp'') <- closure b f (length as) i cd lab' stmp'
 	 return (pcode ++ ccode, lab'', stmp)
 fcompile' (Foreign ty f as) cd lab stmp = foreigncomp ty f as cd lab stmp
 fcompile' (While c sc) cd@(CD mod vars tmp b tr cache ds vs vinfo) lab stmp
@@ -536,26 +538,36 @@ fcompile' (TryCatch tr ca (Loc err) fin) cd@(CD mod vars tmp b trys cache ds vs
    where catchcode = MN ("l",lab)
 	 okay = MN ("l",lab+1)
 	 lab' = lab+2
+-- Any variable which might be modified within the try block (conservatively,
+-- anything which is used at all) must be put on the *heap* before entry,
+-- because if there is an exception and longjmp restores the stack state, we
+-- want the *new* value, not the value at which the try block was entered.
 fcompile' (NewTryCatch tr cs) cd@(CD mod vars tmp b trys cache ds vs vinfo) lab stmp = 
         do let cd' = (CD mod vars tmp b (trys+1) cache ds vs vinfo)
+           let used = locsUsed tr
+           let heapvals = catMaybes $ map (mkHeapVal vars) used
 	   (trcode, lab'', stmp') <- fcompile' tr cd' lab' stmp
 	   (catchblocks, lab''', stmp'') <- catchcompile cs okay cd lab'' stmp'
 	   let tcode = ((TRY catchcode):trcode) ++ [TRIED, JUMP okay] ++
 		       (LABEL catchcode):catchblocks ++ [LABEL okay]
-	   return (tcode, lab''', stmp)
+	   return (heapvals ++ tcode, lab''', stmp)
    where catchcode = MN ("l",lab)
 	 okay = MN ("l",lab+1)
 	 lab' = lab+2
+         -- Don't do variables not in scope yet!
+         mkHeapVal vars i | i<(length vars) = Just $ HEAPVAL (vars!>i)
+                          | otherwise = Nothing
 fcompile' (Throw err) cd lab stmp = 
         do (errcode, lab', stmp') <- fcompile' err cd lab stmp
-	   return (errcode ++ [THROW], lab', stmp)
+	   return (remember (varargs cd) ++ errcode ++ [THROW], lab', stmp)
 fcompile' (Except err code) cd lab stmp = 
         do (errcode, lab', stmp') <- fcompile' err cd lab stmp
 	   (ccode, lab'', stmp'') <- fcompile' code cd lab' stmp'
 	   return (errcode ++ ccode ++ [MKEXCEPT], lab'', stmp)
 fcompile' (Break f l) cd lab stmp = case (breaklabel cd) of
-			    Nothing -> fail $ f++":"++show l++":"++
-				       "Can't break out of function - use 'return'"
+                            Nothing -> fcompile' VoidReturn cd lab stmp
+{-			    Nothing -> fail $ f++":"++show l++":"++
+				       "Can't break out of function - use 'return'" -}
 			    Just l -> return ([JUMP l],lab,stmp)
 fcompile' (PrintNum x) cd lab stmp
      = do (code, lab',stmp') <- fcompile' x cd lab stmp
@@ -585,7 +597,7 @@ fcompile' (CmpExcept op x y) cd lab stmp =
 	 cmpcode NEqual = [NEEXCEPT]
 fcompile' (CmpStr op x y) cd lab stmp = 
    do (xcode, lab', stmp') <- fcompile' x cd lab stmp
-      (ycode, lab'', stmp'') <- fcompile' y cd lab' stmp
+      (ycode, lab'', stmp'') <- fcompile' y cd lab' stmp'
       return (xcode ++ ycode ++ cmpcode op, lab'', stmp)
    where cmpcode Equal = [EQSTRING]
 	 cmpcode NEqual = [NESTRING]
@@ -594,13 +606,15 @@ fcompile' (Append x y) cd lab stmp =
       (ycode, lab'', stmp'') <- fcompile' y cd lab' stmp'
       return (xcode ++ ycode ++ [APPEND], lab'', stmp)
 fcompile' (AppendChain xs) cd lab stmp = 
-   do (chcode, lab', stmp') <- chain xs cd lab stmp
-      return ((PUSH EMPTYSTR):chcode, lab', stmp)
+   do (chcode, lab', stmp') <- chain xs cd lab (stmp+1)
+      return ((USETMP (tmpvar stmp)):(STACKSTR (tmpvar stmp) (EMPTYSTR)):chcode, lab', (stmp+1))
     where chain (x:xs) cd lab stmp
               = do (xcode, lab',stmp') <- fcompile' x cd lab stmp
                    (xscode, lab'',stmp'') <- chain xs cd lab' stmp'
-                   return (xcode ++ APPENDTOP:xscode, lab'',stmp)
+                   return (xcode ++ APPENDTOP:xscode, lab'',stmp'')
           chain [] cd lab stmp = return ([], lab,stmp)
+          tmpvar s = ((MN ("stmp",s)),True)
+
 fcompile' (Unary op x) cd lab stmp = icompile False (Unary op x) cd lab stmp
 fcompile' (RealUnary op x) cd lab stmp = rcompile (RealUnary op x) cd lab stmp
 fcompile' (Coerce t1 t2 v) cd lab stmp = 
@@ -620,7 +634,9 @@ fcompile' (Field v n a t) cd lab stmp =
       return (vcode ++ [PROJARG a t], lab', stmp)
 fcompile' (ArrayInit xs) cd lab stmp =
    do (xscode, lab', stmp') <- pushargs xs cd lab stmp
-      return (xscode ++ [MKARRAY (length xs)], lab', stmp)
+      return (xscode ++ [USETMP (tmpvar stmp'), MKARRAY (tmpvar stmp') (length xs)], lab', (stmp'+1))
+   where tmpvar s = ((MN ("stmp",s)),True)
+
 
 --fcompile' (GConst (Num i)) cd lab
 --   where i = getFunID n (alldecls cd)
@@ -731,10 +747,10 @@ ccompile_stack (Bo False) stmp lab
    where tmpvar = ((MN ("stmp",stmp)),True)
 --    = return ([PUSH (VAL 0)], lab)
 ccompile_stack (Str str) stmp lab 
-    = return ([PUSH (STR str)], lab, stmp)
+--    = return ([PUSH (STR str)], lab, stmp)
 -- doesn't quite work yet
-{-    = return ([USETMP tmpvar, STACKSTR tmpvar (STR str)], lab, stmp+1)
-   where tmpvar = ((MN ("stmp",stmp)),True) -}
+    = return ([USETMP tmpvar, STACKSTR tmpvar (STR str)], lab, stmp+1)
+   where tmpvar = ((MN ("stmp",stmp)),True) 
 ccompile_stack (Exc str i) stmp lab 
     = return ([PUSH (STR str), PUSH (VAL i), MKEXCEPT], lab, stmp)
 
@@ -784,11 +800,11 @@ tailcall (Loc i) cd lab stmp = return ([TAILCALL ((locs cd)!>i)],lab,stmp)
 tailcall x cd lab stmp = do (ccode,lab',stmp') <- fcompile' x cd lab stmp
 		            return ((ccode ++ [TAILCALLTOP]),lab',stmp')
 
-closure (Global n m ar) i left cd lab stmp
-    = return ([CLOSURE (cname n m) i ar],lab,stmp)
-closure (Loc l) i left cd lab stmp = return ([CLOSURELOC ((locs cd)!>l) i],lab,stmp)
-closure x i left cd lab stmp = do (ccode,lab',stmp') <- fcompile' x cd lab stmp
-		                  return ((ccode ++ [CLOSURETOP i]),lab',stmp)
+closure b (Global n m ar) i left cd lab stmp
+    = return ([CLOSURE b (cname n m) i ar],lab,stmp)
+closure b (Loc l) i left cd lab stmp = return ([CLOSURELOC b ((locs cd)!>l) i],lab,stmp)
+closure b x i left cd lab stmp = do (ccode,lab',stmp') <- fcompile' x cd lab stmp
+		                    return ((ccode ++ [CLOSURETOP b i]),lab',stmp)
 
 {-
 closure x cd lab = let (ccode,lab') = fcompile' x cd lab in
@@ -844,7 +860,7 @@ icompile hp e cd@(CD mod vars tmp b tr cache ds vs vinfo) lab stmp =
      icomp' (GConst (Num x)) lab tmp stmp = return ([TMPSET tmp x], lab, stmp)
      icomp' x lab tmp stmp = 
 	  do (xcode, lab',stmp') <- fcompile' x (CD mod vars (tmp+1) b tr cache ds vs vinfo) lab stmp
-	     return (xcode ++ [GETVAL tmp], lab',stmp)
+	     return (xcode ++ [GETVAL tmp], lab',stmp')
 
 rcompile :: Monad m => Expr Name -> CompData -> Int -> Int -> m ([TAC], Int, Int)
 rcompile e cd@(CD mod vars tmp b tr cache ds vs vinfo) lab stmp = 
@@ -878,26 +894,40 @@ rcompile e cd@(CD mod vars tmp b tr cache ds vs vinfo) lab stmp =
 	 do (xcode, lab',stmp') <- fcompile' x (CD mod vars (tmp+1-10000) b tr cache ds vs vinfo) lab stmp
 	    return (xcode ++ [GETRVAL tmp], lab', True, stmp)
 
+cocompile' :: Monad m => [TAC] -> Int -> Int -> m([TAC], Int, Int)
+cocompile' vcode lab stmp = 
+    do return ([USETMP tmpvar, STACKINT tmpvar 0]++vcode, lab, (stmp+1))
+           where tmpvar = ((MN ("stmp",stmp)),True)
+
 cocompile :: Monad m => Type -> Type -> [TAC] -> CompData -> Int -> Int ->
 	                m ([TAC], Int, Int)
 cocompile (Prim StringType) (Prim Number) vcode cd lab stmp =
-       return (vcode ++ [STR2INT], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [STR2INT], lab', stmp')
 cocompile (Prim Number) (Prim StringType) vcode cd lab stmp =
-       return (vcode ++ [INT2STR], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [INT2STR], lab, stmp')
 cocompile (Prim RealNum) (Prim StringType) vcode cd lab stmp =
-       return (vcode ++ [REAL2STR], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [REAL2STR], lab, stmp')
 cocompile (Prim StringType) (Prim RealNum) vcode cd lab stmp =
-       return (vcode ++ [STR2REAL], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [STR2REAL], lab, stmp')
 cocompile (Prim StringType) (Prim Character) vcode cd lab stmp =
-       return (vcode ++ [STR2CHR], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [STR2CHR], lab, stmp')
 cocompile (Prim Character) (Prim StringType) vcode cd lab stmp =
-       return (vcode ++ [CHR2STR], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [CHR2STR], lab, stmp')
 cocompile (Prim Boolean) (Prim StringType) vcode cd lab stmp =
-       return (vcode ++ [BOOL2STR], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [BOOL2STR], lab, stmp')
 cocompile (Prim Number) (Prim RealNum) vcode cd lab stmp =
-       return (vcode ++ [INT2REAL], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [INT2REAL], lab, stmp')
 cocompile (Prim RealNum) (Prim Number) vcode cd lab stmp =
-       return (vcode ++ [REAL2INT], lab, stmp)
+    do (vcode', lab', stmp') <- cocompile' vcode lab stmp
+       return (vcode' ++ [REAL2INT], lab, stmp')
 cocompile (Prim Character) (Prim Number) vcode cd lab stmp = return (vcode, lab, stmp)
 cocompile (Prim Number) (Prim Character) vcode cd lab stmp = return (vcode, lab, stmp)
 cocompile _ _ v _ _ _ = fail "Internal error, can't coerce"
@@ -1017,19 +1047,12 @@ annotcompile (LamBody f) code lab stmp
 annotcompile (DynCheck t) code lab stmp = return (code,lab,stmp)
 
 decls :: [TAC] -> [TAC]
-decls c = add_decls (find_decls c ([],[],[],[],[])) c
+decls c = add_decls (find_decls c) c
 
-add_decls (ts,ts',vs,vs',as) c 
-    = (map TMPINT (removedups ts)) ++
-      (map TMPREAL (removedups ts')) ++
-      (map dodec (removedups (removeothers vs' vs))) ++ 
-      (map DECLAREQUICK (removedups vs')) ++
---      (map DECLAREARG (removedups vs')) ++
-      (map ARRAY (removedups as)) ++ c
-
-dodec (v,True) = DECLAREQUICK (v, True)
-dodec (v,False) = DECLAREQUICK (v, False)
---dodec (v,False) = DECLARE (v, False)
+add_decls (is,fs,vs) c 
+    = (map TMPINT is) ++
+      (map TMPREAL fs) ++
+      (map DECLAREQUICK vs) ++ c
 
 removedups [] = []
 removedups (x:xs) | x `elem` xs = removedups xs
@@ -1039,66 +1062,135 @@ removeothers vs [] = []
 removeothers vs (x:xs) | x `elem` vs = removeothers vs xs
 		       | otherwise = x:(removeothers vs xs)
 
--- FIXME: Yeeeuuch! There are much neater ways of doing this, probably with
--- a State monad.
+-- Find variable names for: unboxed integers, floats and heap Values so that
+-- we can declare them in the C output
 
-find_decls :: [TAC] -> ([Int],[Int],[Var],[Var],[Var]) -> 
-	               ([Int],[Int],[Var],[Var],[Var])
-find_decls [] (ts,ts',vs,vs',as) = (ts,ts',vs,vs',as)
-find_decls ((SET x i y):xs) (ts,ts',vs,vs',as) 
-    = find_decls xs (ts,ts',x:y:vs,vs',as)
+type DeclState = ([Int], [Int], [Var])
+
+addTmpInt :: Int -> State DeclState ()
+addTmpInt x = do (is,fs,vs) <- get
+                 if (x `elem` is) 
+                    then return ()
+                    else put (x:is, fs, vs)
+
+addTmpFloat :: Int -> State DeclState ()
+addTmpFloat x = do (is,fs,vs) <- get
+                   if (x `elem` fs) 
+                    then return ()
+                    else put (is, x:fs, vs)
+
+addHeapVar :: Var -> State DeclState ()
+addHeapVar x = do (is,fs,vs) <- get
+                  if (x `elem` vs) 
+                    then return ()
+                    else put (is, fs, x:vs)
+
+find_decls :: [TAC] -> DeclState
+find_decls code = execState (find_declsSt code) ([],[],[])
+
+find_declsSt :: [TAC] -> State DeclState ()
+find_declsSt [] = return ()
+find_declsSt ((SET x i y):xs)
+    = do addHeapVar x
+         addHeapVar y
+         find_declsSt xs
 --find_decls ((USETMP v):xs) (ts,ts',vs,vs',as) 
 --    = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((TMPSET x i):xs) (ts,ts',vs,vs',as) 
-    = find_decls xs (x:ts,ts',vs,vs',as)
-find_decls ((GETVAL x):xs) (ts,ts',vs,vs',as) 
-    = find_decls xs (x:ts,ts',vs,vs',as)
-find_decls ((GETRVAL x):xs) (ts,ts',vs,vs',as) 
-    = find_decls xs (ts,x:ts',vs,vs',as)
-find_decls ((RTMPSET x i):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,x:ts',vs,vs',as)
-find_decls ((CALL v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((TAILCALL v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((INFIX t1 _ t2 t3):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (t1:t2:t3:ts,ts',vs,vs',as)
-find_decls ((INTPOWER t1 t2 t3):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (t1:t2:t3:ts,ts',vs,vs',as)
-find_decls ((UNARY t1 _ t2):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (t1:t2:ts,ts',vs,vs',as)
-find_decls ((REALINFIX t1 _ t2 t3):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (ts,t1:t2:t3:ts',vs,vs',as)
-find_decls ((REALINFIXBOOL _ t2 t3):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (ts,t2:t3:ts',vs,vs',as)
-find_decls ((REALPOWER t1 t2 t3):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (ts,t1:t2:t3:ts',vs,vs',as)
-find_decls ((REALUNARY t1 _ t2):xs) (ts,ts',vs,vs',as) 
-   = find_decls xs (ts,t1:t2:ts',vs,vs',as)
-find_decls ((PUSH (VAR v)):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((PUSHSETTOP (VAR v)):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((PUSHGETVAL (VAR v) t):xs) (ts,ts',vs,vs',as) = find_decls xs (t:ts,ts',v:vs,vs',as)
-find_decls ((PUSHGETRVAL (VAR v) t):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,t:ts',v:vs,vs',as)
-find_decls ((PUSHGETINDEX (VAR v)):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((PUSHTOINDEX (VAR v)):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((SETLOOKUP _ v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((POP v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((PUSHSETINT (VAR v) x):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((SETINT v x):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((SETVAR v x):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:x:vs,vs',as)
-find_decls ((POPARG v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',vs,v:vs',as)
-find_decls ((POPANDCOPYARG v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',vs,v:vs',as)
-find_decls ((USETMP v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',vs,v:vs',as)
-find_decls ((POPINDEX v):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',v:as)
-find_decls ((CASE alts):xs) (ts,ts',vs,vs',as) = find_decls xs 
-					    (fd_alts alts (ts,ts',vs,vs',as))
-   where fd_alts [] acc = acc
-	 fd_alts (xs:xss) acc = find_decls xs (fd_alts xss acc)
-find_decls ((CONSTCASE _ ccs defcode):xs) (ts,ts',vs,vs',as) 
-    = find_decls xs (find_decls defcode 
-                     (fd_alts (map snd ccs) (ts,ts',vs,vs',as)))
-   where fd_alts [] acc = acc
-	 fd_alts (xs:xss) acc = find_decls xs (fd_alts xss acc)
-find_decls ((PROJ v i):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls ((EXPROJARG v i):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
-find_decls (_:xs) acc = find_decls xs acc
+find_declsSt ((TMPSET x i):xs)
+    = do addTmpInt x
+         find_declsSt xs
+find_declsSt ((GETVAL x):xs)
+    = do addTmpInt x
+         find_declsSt xs
+find_declsSt ((GETRVAL x):xs)
+    = do addTmpFloat x
+         find_declsSt xs
+find_declsSt ((RTMPSET x i):xs)
+    = do addTmpFloat x
+         find_declsSt xs
+find_declsSt ((CALL v):xs) 
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((TAILCALL v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((INFIX t1 _ t2 t3):xs)
+    = do addTmpInt t1 ; addTmpInt t2 ; addTmpInt t3
+         find_declsSt xs
+find_declsSt ((INTPOWER t1 t2 t3):xs)
+    = do addTmpInt t1 ; addTmpInt t2 ; addTmpInt t3
+         find_declsSt xs
+find_declsSt ((UNARY t1 _ t2):xs)
+    = do addTmpInt t1 ; addTmpInt t2
+         find_declsSt xs
+find_declsSt ((REALINFIX t1 _ t2 t3):xs)
+    = do addTmpFloat t1 ; addTmpFloat t2 ; addTmpFloat t3
+         find_declsSt xs
+find_declsSt ((REALINFIXBOOL _ t2 t3):xs)
+    = do addTmpFloat t2 ; addTmpFloat t3
+         find_declsSt xs
+find_declsSt ((REALPOWER t1 t2 t3):xs)
+    = do addTmpFloat t1 ; addTmpFloat t2 ; addTmpFloat t3
+         find_declsSt xs
+find_declsSt ((REALUNARY t1 _ t2):xs)
+    = do addTmpFloat t1 ; addTmpFloat t2
+         find_declsSt xs
+find_declsSt ((PUSH (VAR v)):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((PUSHSETTOP (VAR v)):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((PUSHGETVAL (VAR v) t):xs)
+    = do addHeapVar v ; addTmpInt t
+         find_declsSt xs
+find_declsSt ((PUSHGETRVAL (VAR v) t):xs)
+    = do addHeapVar v ; addTmpFloat t
+         find_declsSt xs
+find_declsSt ((PUSHGETINDEX (VAR v)):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((PUSHTOINDEX (VAR v)):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((SETLOOKUP _ v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((POP v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((PUSHSETINT (VAR v) x):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((SETINT v x):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((SETVAR v x):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((POPARG v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((POPANDCOPYARG v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((USETMP v):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((CASE alts):xs) 
+    = do find_declsSt xs
+         mapM_ find_declsSt alts
+find_declsSt ((CONSTCASE _ ccs defcode):xs)
+    = do find_declsSt xs
+         find_declsSt defcode
+         mapM_ find_declsSt (map snd ccs)
+find_declsSt ((PROJ v i):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt ((EXPROJARG v i):xs)
+    = do addHeapVar v
+         find_declsSt xs
+find_declsSt (_:xs) = find_declsSt xs
 
 
 -- compileDecls :: [ConDecl] -> [CompileResult]
@@ -1154,7 +1246,7 @@ compileAll ds n = ca' ds n
 	return $ (RawCode ("extern \"C\" {\n#include <"++str++">\n}")):rest
     ca' (FMName n:xs) mod = do
         rest <- (ca' xs mod)
-        return $ (RawCode ("void " ++ n ++ "(VMState* vm);\n")):rest
+        return $ (RawCode ("extern \"C\" { void " ++ n ++ "(VMState* vm);\n } ")):rest
     ca' ((FunBind (_,_,n,ty,fopts,Defined e) _ _):xs) mod = do
         rest <- ca' xs mod
 	code <- comp ds e (elem Pure fopts) mod
@@ -1216,16 +1308,27 @@ peephole ((STACKTMP a i):(JTRUE l):xs) = (JTTRUE i l):(peephole xs)
 peephole ((PUSH x):SETTOP:xs) = (PUSHSETTOP x):(peephole xs)
 peephole ((PUSH (INT x)):(JFALSE l):xs) = (JTFALSE x l):(peephole xs)
 peephole ((PUSH (INT x)):(JTRUE l):xs) = (JTTRUE x l):(peephole xs)
+-- FIXME: These three are probably not best done as peephole optimisations
+-- better to do it before Coerce gets evaluated at all if possible.
 peephole (CHR2STR:APPENDTOP:xs) = APPENDTOPINT:(peephole xs)
 peephole ((PUSH (VAL x)):CHR2STR:APPENDTOP:xs) = (APPENDCHAR x):(peephole xs)
 peephole ((PUSH (INT x)):CHR2STR:APPENDTOP:xs) = (APPENDTMP x):(peephole xs)
 peephole ((PUSH (STR x)):APPENDTOP:xs) = (APPENDSTR x):(peephole xs)
+peephole ((STACKSTR v (STR x)):APPENDTOP:xs) = (APPENDSTR x):(peephole xs)
+-- various string tests
 peephole ((PUSH (STR x)):EQSTRING:(JFALSE l):xs) = (JNESTRINGW l x):(peephole xs)
 peephole ((PUSH (STR x)):EQSTRING:(JTRUE l):xs) = (JEQSTRINGW l x):(peephole xs)
 peephole ((PUSH (STR x)):NESTRING:(JFALSE l):xs) = (JEQSTRINGW l x):(peephole xs)
 peephole ((PUSH (STR x)):NESTRING:(JTRUE l):xs) = (JNESTRINGW l x):(peephole xs)
 peephole ((PUSH (STR x)):EQSTRING:xs) = (EQSTRINGW x):(peephole xs)
 peephole ((PUSH (STR x)):NESTRING:xs) = (NESTRINGW x):(peephole xs)
+-- and the same with stack strings
+peephole ((STACKSTR v (STR x)):EQSTRING:(JFALSE l):xs) = (JNESTRINGW l x):(peephole xs)
+peephole ((STACKSTR v (STR x)):EQSTRING:(JTRUE l):xs) = (JEQSTRINGW l x):(peephole xs)
+peephole ((STACKSTR v (STR x)):NESTRING:(JFALSE l):xs) = (JEQSTRINGW l x):(peephole xs)
+peephole ((STACKSTR v (STR x)):NESTRING:(JTRUE l):xs) = (JNESTRINGW l x):(peephole xs)
+peephole ((STACKSTR v (STR x)):EQSTRING:xs) = (EQSTRINGW x):(peephole xs)
+peephole ((STACKSTR v (STR x)):NESTRING:xs) = (NESTRINGW x):(peephole xs)
 peephole (EQSTRING:(JFALSE l):xs) = (JNESTRING l):(peephole xs)
 peephole (EQSTRING:(JTRUE l):xs) = (JEQSTRING l):(peephole xs)
 peephole (NESTRING:(JFALSE l):xs) = (JEQSTRING l):(peephole xs)
@@ -1253,23 +1356,6 @@ peephole ((STACKINT a _):(JTRUE l):xs) = (JUMP l):(peephole xs)
 -- optimise return values
 peephole ((PUSH (REAL r)):SETRV:xs) = (SETRVREAL r):(peephole xs)
 
--- Slightly trickier string-in-place optimisations
-{-
-peephole ((PUSH x1):(PUSH (STR y)):APPEND:(PUSH x2):SETTOP:xs) 
-         | x1 == x2 = (PUSH x1):(APPENDSTR y):DISCARD:(peephole xs)
-peephole ((PUSH x1):(PUSH (VAL y)):CHR2STR:APPEND:(PUSH x2):SETTOP:xs) 
-         | x1 == x2 = (PUSH x1):(APPENDCHAR y):DISCARD:(peephole xs)
-peephole ((PUSH x1):(PUSH (INT y)):CHR2STR:APPEND:(PUSH x2):SETTOP:xs) 
-         | x1 == x2 = (PUSH x1):(APPENDTMP y):DISCARD:(peephole xs)
--}
-
---peephole ((POPARG x):(PUSH (VAR y)):xs) | x==y = peephole xs
-{-
-peephole ((PUSH x):(PUSH y):(PUSH z):(PUSH w):xs) 
-    = (PUSH4 x y z w):(peephole xs)
-peephole ((PUSH x):(PUSH y):(PUSH z):xs) = (PUSH3 x y z):(peephole xs)
-peephole ((PUSH x):(PUSH y):xs) = (PUSH2 x y):(peephole xs)
--}
 peephole ((PUSH d@(VAL v)):(PUSH s):(PROJARG a t):SETTOP:xs) = (SETPROJARG s a t d):(peephole xs)
 
 peephole ((SET v 0 x):xs) = (SETVAR v x):(peephole xs)
@@ -1336,36 +1422,6 @@ foreigncall ty (UN n) args = [USETMP (tmpval n) | n <- [0..(length args)-1]]
 foreigncall ty (UN n) args = [USETMP ((tmpval n), True) | n <- [0..(length args)-1]]
 			     ++ [FOREIGNCALL n "nolibsyet" ty args]
 
-{-   where conv (Prim Void) = "VOID("
-	 conv (Prim Number) = "PUSH(MKINT"
-	 conv (Prim RealNum) = "PUSH(MKREAL"
-	 conv (Prim Boolean) = "PUSH(MKINT"
-	 conv (Prim Character) = "PUSH(MKCHAR"
-	 conv (Prim StringType) = "PUSH(MKSTR"
-	 conv (Prim Pointer) = "PUSH(MKINT"
-	 conv (TyVar _) = "PUSH(" -- Enough rope to hang yourself with!
-	 conv (Array _) = "PUSH(MKARRAYVAL"
-	 conv t = error $ "Can't deal with that type in foreign calls" ++ show t
-
-         popvals n [] = ""
-	 popvals n (x:xs) = show (tmpval n) ++ 
-			    " = vm->doPop(); " ++ popvals (n+1) xs
-
-         stackconv n [] = ""
-	 stackconv n [x] = stackconv' n x
-	 stackconv n (x:xs) = stackconv' n x ++ "," ++ stackconv (n+1) xs
-	 stackconv' n (Prim Number) = show (tmpval n) ++ "->getInt()"
-	 stackconv' n (Prim RealNum) = show (tmpval n) ++ "->getReal()"
-	 stackconv' n (Prim Boolean) = show (tmpval n) ++ "->getInt()"
-	 stackconv' n (Prim Character) = show (tmpval n) ++ "->getInt()"
-	 stackconv' n (Prim StringType) 
-	     = show (tmpval n) ++ "->getString()->getVal()"
-	 stackconv' n (Prim Pointer) = show (tmpval n) ++ "->getRaw()"
-	 stackconv' n (Array _) = show (tmpval n) ++ "->getArray()"
-	 stackconv' n (TyVar _) = show (tmpval n)
-	 
-	 stackconv' _ t = error $ "Can't deal with that type (" ++ show t ++ ") in foreign calls"
--}
 
 mkStartup :: Program -> CompileResult
 mkStartup p = let startups = callStartups p in
