@@ -1,8 +1,8 @@
 #include "curses_glue.h"
-#include <ncurses.h>
+#include <curses.h>
 
-bool testerr(int x) {
-    return x == ERR;
+bool testerr(void* x) {
+    return (kint)x == ERR;
 }
 
 void docbreak() { cbreak(); }
@@ -14,8 +14,12 @@ void dononl() { nonl(); }
 void doraw() { raw(); }
 void donoraw() { noraw(); }
 
-WINDOW* getstdscr() {
-    return stdscr;
+void* getstdscr() {
+    return (void*)stdscr;
+}
+
+void* doinitscr() {
+  return (void*)initscr();
 }
 
 void dointrflush(void* v, bool bf) {
@@ -23,7 +27,9 @@ void dointrflush(void* v, bool bf) {
 }
 
 void dokeypad(void* v, bool bf) {
+#ifndef WIN32
     keypad((WINDOW*)v,bf);
+#endif
 }
 
 void dometa(void* v, bool bf) {
@@ -31,7 +37,9 @@ void dometa(void* v, bool bf) {
 }
 
 void donodelay(void* v, bool bf) {
+#ifndef WIN32
     nodelay((WINDOW*)v,bf);
+#endif
 }
 
 void donotimeout(void* v, bool bf) {
@@ -271,6 +279,13 @@ DEFKEYF(10);
 DEFKEYF(11);
 DEFKEYF(12);
 
+void* docurs_set(int s) {
+#ifndef WIN32
+    return (void*)curs_set(s);
+#else
+    return (void*)0;
+#endif  
+}
 
 int dobox(void* win,wchar_t x, wchar_t y)
 {
