@@ -1,9 +1,9 @@
 #ifndef _CURSES_GLUE_H
 #define _CURSES_GLUE_H
 
-#include <ncurses.h>
 #include <KayaAPI.h>
 #include <stdfuns.h>
+#include <curses.h>
 
 #define DECLKEY(x) wchar_t key_## x();
 #define DEFKEY(x) wchar_t key_## x() { return KACHAR(KEY_## x); }
@@ -12,7 +12,7 @@
 
 
 extern "C" {
-    bool testerr(int x);
+    bool testerr(void* x);
 
     void docbreak();
     void donocbreak();
@@ -22,8 +22,9 @@ extern "C" {
     void dononl();
     void doraw();
     void donoraw();
+    void* doinitscr();
 
-    WINDOW* getstdscr();
+    void* getstdscr();
     void dointrflush(void* v, bool bf);
     void dokeypad(void* v, bool bf);
     void dometa(void* v, bool bf);
@@ -97,6 +98,8 @@ extern "C" {
 
     int dobox(void* win,wchar_t x, wchar_t y);
 
+    void* docurs_set(int s);
+
     void doclear();
     void doerase();
     void doclrtobot();
