@@ -3,6 +3,7 @@
 #include <VMState.h>
 #include <cstdlib>
 #include <KayaAPI.h>
+#include <cstring>
 
 void* pg_connect(wchar_t* rawconninfo)
 {
@@ -93,6 +94,10 @@ void* pg_exec(void* vmptr,void* conn,wchar_t* rawquery, KayaValue err)
     int numrows = PQntuples(res);
     int numflds = PQnfields(res);
     
+    KayaArray col_names = newKayaArray(numflds);
+    for (int i = 0; i<numflds; i++)
+	KayaArrayPush(col_names, KayaString(strtowc(PQfname(res,i))));
+
     KayaArray resarray = newKayaArray(numrows);
 
     for(int i = 0; i<numrows; i++) {
@@ -146,21 +151,11 @@ void* pg_exec(void* vmptr,void* conn,wchar_t* rawquery, KayaValue err)
     pgr -> res_table = resarray;
     pgr -> rows = numrows;
     pgr -> cols = numflds;
-    pgr -> pgres = res;
+    pgr -> col_names = col_names;
     PQclear(res);
     return pgr;
 }
 
-wchar_t* pg_columnname(void* vmptr,void* res, int col)
-{
-    VMState* vm = (VMState*)vmptr;
-    PGresult *r = ((PGRes*)res)->pgres;
-    if (col>=((PGRes*)res)->cols) {
-	vm->kaya_internalError(3);
-    }
-    return strtowc(PQfname(r,col));
-}
-
 Array* pg_getstrs(void* res)
 {
     return ((PGRes*)res)->res_table;
@@ -176,6 +171,11 @@ int pg_numcols(void* res)
     return ((PGRes*)res)->cols;
 }
 
+Array* pg_colnames(void* res)
+{
+    return ((PGRes*)res)->col_names;
+}
+
 void pg_close(void* conn)
 {
     PQfinish(((PGCon*)conn)->con);
@@ -287,7 +287,6 @@ void* pg_execp(void* vmptr,void* conn,void* queryptr, KayaValue err, KArray para
     pgr -> res_table = resarray;
     pgr -> rows = numrows;
     pgr -> cols = numflds;
-    pgr -> pgres = res;
     PQclear(res);
     return pgr;
 }
