@@ -17,6 +17,7 @@ extern "C" {
 
   void* do_waitevent();
   kint event_type(void* eptr);
+  kint do_getkeymod(void* eptr);
   kint do_getkeynum(void* eptr);
   kint do_getmmx(void* eptr);
   kint do_getmmy(void* eptr);
@@ -30,6 +31,7 @@ extern "C" {
 		   kint r, kint g, kint b, kint a);
 
   void do_setwmtitles(wchar_t* window, wchar_t* icon);
+  void do_seticon(void* icon);
 
 }
 
