@@ -33,6 +33,7 @@ public:
     int rows;
     int cols;
     int res;
+    sqlite3_stmt* resptr;
 };
 
 extern "C" {
@@ -41,6 +42,9 @@ extern "C" {
     wchar_t* sqlite_getError(void* con);
     void sqlite_close(void* con);
     void* sqlite_exec(void* con, wchar_t* query);
+    void* sqlite_incexec(void* con, wchar_t* query);
+    KArray sqlite_getrow(void* vmptr, void* con);
+    void sqlite_discard(void* sptr);
 
     void* sqlite_execp(void* stptr, KArray params);
     void* sqlite_prepare(void* vmptr, void* con, wchar_t* query);
