@@ -10,7 +10,11 @@ void* threadFn(void* vfn)
     // Make new vm data, push the argument, call the function.
     VMState* vm = initstack();
 //    cout << "Making new vm " << vm << endl;
-    PUSH(ts->arg);
+    PUSH(new Value(NULL,KVT_NULL)); // return value
+    if (ts->arg!=NULL) {
+	PUSH(ts->arg);
+    }
+
     CALLFUN(ts->fn);
     return vm->doPop();
 }
@@ -28,6 +32,11 @@ void* createThread(Value* fn,Value* arg)
     return (void*)td;
 }
 
+void* createThreadNullary(Value* fn)
+{
+    createThread(fn, NULL);
+}
+
 Value* waitForThread(void* tid)
 {
     ThreadData* td = (ThreadData*)tid;
