@@ -24,7 +24,8 @@ class Closure : public gc_cleanup {
 public:
     // Create a closure from a function, which will have <args> as its locals,
     // and space for <arity> arguments.
-    Closure(VMState* vm, func fn,kint args, kint arity, bool stack=true);
+    Closure(VMState* vm, func fn,kint args, kint arity, 
+	    bool lam, bool stack=true);
     Closure(Closure& x);
     ~Closure();
 
@@ -38,10 +39,10 @@ public:
     Value** getArgs() { return m_args; }
 
     void setArg(kint i,Value* v) { m_args[i]=v; }
-    void addArgs(VMState* vm, kint i);
+    void addArgs(VMState* vm, kint i, bool lam);
 
     bool eq(Closure* c,map<Value*,Value*>& done);
-    Closure* copy(map<Value*, Value*>& done);
+    Closure* copy(VMState* vm, map<Value*, Value*>& done);
 
 private:
     func m_fn;
