@@ -54,6 +54,11 @@ KValue KayaString(wchar_t* str)
     return v;
 }
 
+KValue KayaString2(VMState* vm)
+{
+    return new Value(new String(vm),KVT_STRING);
+}
+
 KValue KayaCharString(char* str)
 {
     KValue v = newKayaValue();
@@ -112,6 +117,13 @@ KArray newKayaArray(kint size)
     return new Array(size);
 }
 
+KArray newKayaArray2(void* vmptr, kint size)
+{
+    VMState* vm = (VMState*) vmptr;
+    return new(vm->getAPool()) Array(size);
+}
+
+
 KArray KayaGetArray(KValue v)
 {
     return v->getArray();
