@@ -37,6 +37,7 @@ typedef double KFloat;
 /// String conversion macros
 // char* to KString
 #define KSTRING(x) strtowc(x)
+#define KVSTRING(vm,x) strtovwc(vm,x)
 // char to KChar
 #define KCHAR(x) chrtowc(x)
 // curses alt-chars to KChar
@@ -44,6 +45,7 @@ typedef double KFloat;
 // KString to char*
 #define CSTRING(x) wctostr(x)
 #define CNSTRING(x,i) wcntostr(x,i)
+#define CNPSTRING(narrow,wide,len) char narrow[(len+1)*4]; wcnptostr(wide,len,narrow);
 // KChar to char
 #define CCHAR(x) wctochr(x)
 
@@ -61,6 +63,8 @@ extern "C" {
     /// Make a new, empty array, with room for <size> elements.
     /// Give size 0 to let the rts choose the size.
     KArray newKayaArray(kint size);
+  // same as above, but with the VM pointer supplied
+    KArray newKayaArray2(void* vmptr, kint size);
 
     // Make new, initialised values.
     KValue KayaInt(KInt x);
@@ -68,6 +72,7 @@ extern "C" {
     KValue KayaChar(KChar c);
     KValue KayaFloat(KFloat f);
     KValue KayaString(KString str);
+    KValue KayaString2(VMState* vm);
     KValue KayaCharString(char* str);
     KValue KayaUnion(kint tag, kint args);
     KValue KayaArrayVal(KArray ar);
