@@ -11,12 +11,13 @@
 
 using namespace std;
 
+#include "Heap.h"
+
 #include <setjmp.h>
 #include <math.h>
 #include <wchar.h>
 
 //#include "stdfuns.h"
-#include "Heap.h"
 #include "Array.h"
 #include "ValueFuns.h"
 #include "Closure.h"
@@ -32,29 +33,29 @@ class VMState;
 #define DECLARE(x) Value *x = NEWVAL; x->setInt(0);
 // Args get initialised immediately anyway
 #define DECLAREARG(x) Value stack_##x(NULL,KVT_INT);
-#define DECLAREQUICK(x) Value stack_##x(NULL,KVT_INT); Value* x = &stack_##x;;
-#define HEAPVAL(x) if (x==&stack_##x) { x = MKVAL(stack_##x.getRaw(), stack_##x.getFunTable()); }
+#define DECLAREQUICK(x) Value stack_##x(NULL,KVT_INT); Value* x = &stack_##x;
+#define HEAPVAL(x) if (x==&stack_##x) { x = MKPVAL(vm->getVPool(),stack_##x.getRaw(), stack_##x.getFunTable()); }
 // cout << "Made heap val of " << &stack_##x << " -> " << x << endl; cout << x->getRaw() << endl; }
 
 //#define DECLAREQUICK(x) Value *x = new Value(NULL,inttable);
 #define TMPINT(x) int x
 #define TMPREAL(x) double x
 //#define ARRAY(x) x=new Value(ARRAY);
-#define MKINT(x) mkint((void*)(x))
+#define MKINT(x) mkint2(vm,(void*)(x))
 //&(Value((void*)x,inttable))
-#define MKREAL(x) (MKVAL((void*)(NEWREAL(x)),KVT_REAL))
-#define MKCHAR(x) (MKVAL((void*)((int)x),KVT_INT))
-#define MKFUN(x,i) (MKVAL((void*)(new Closure(vm,x,0,i)),KVT_FUNC))
-#define MKSTR(x) mkstr(x)
-#define EMPTYSTR (MKVAL((void*)(NEWSTRING((wchar_t*)NULL)),KVT_STRING))
+#define MKREAL(x) (MKPVAL(vm->getVPool(),(void*)(NEWPREAL(vm->getRPool(),x)),KVT_REAL))
+#define MKCHAR(x) (MKPVAL(vm->getVPool(),(void*)((int)x),KVT_INT))
+#define MKFUN(x,i) (MKPVAL(vm->getVPool(),(void*)(new Closure(vm,x,0,i,false)),KVT_FUNC))
+#define MKSTR(x) mkstr2(vm,x)
+#define EMPTYSTR (MKPVAL(vm->getVPool(),(void*)(NEWPSTRING(vm->getSPool(),(wchar_t*)NULL)),KVT_STRING))
 //doesn't work any more
 //#define MKUNION(x) (MKVAL((void*)x,KVT_UNION))
-#define MKCON(t,a) vm->push(MKVAL(new Union(vm,a),KVT_UNION+t));
-#define MKCONRV(t,a) vm->pushrv((void*)(new Union(vm,a)),KVT_UNION+t);
+#define MKCON(t,a) vm->push(MKPVAL(vm->getVPool(),new(vm->getUPool()) Union(vm,a),KVT_UNION+t));
+#define MKCONRV(t,a) vm->pushrv((void*)(new(vm->getUPool()) Union(vm,a)),KVT_UNION+t);
 #define MKNEWEXCEPT(e,a) vm->pushrv((void*)(new Exception(vm,e,a)),KVT_EXCEPTION)
-#define MKCONZERO(t) { vm->push(MKVAL((void*)t,KVT_CUNION)); }
+#define MKCONZERO(t) { vm->push(MKPVAL(vm->getVPool(),(void*)t,KVT_CUNION)); }
 #define MKCONZERORV(t) { vm->pushrv((void*)t,KVT_CUNION); }
-#define MKEXCEPT vm->push(MKVAL((void*)(new Exception(vm)),KVT_EXCEPTION))
+#define MKEXCEPT vm->push(MKPVAL(vm->getVPool(),(void*)(new Exception(vm)),KVT_EXCEPTION))
 //#define SET(x,i,y) x->setIdxPtr(y,i);
 #define MKARRAYVAL(x) mkarrayval((void*)(x))
 
@@ -62,6 +63,7 @@ class VMState;
 #define TOFIELD(i) vm->goToField(i);
 
 #define MKARRAY(i) vm->mkArray(i)
+#define MKARRAY2(s,i) vm->mkArray(s,i)
 #define CALL(x) (x(vm)); 
 //vm->push(vm->getReturnVal());
 #define CALLFUN(x) (x->runClosure(vm)); 
@@ -72,9 +74,9 @@ class VMState;
 //vm->push(vm->getReturnVal());
 #define TAILCALLTOP (vm->doPop()->runClosure(vm))
 
-#define CLOSURE(x,i,tot) vm->push(MKVAL((void*)(new Closure(vm,x,i,tot)),KVT_FUNC))
-#define CLOSURELOC(x,i) vm->push(x->clone()->addClosureArgs(vm,i));
-#define CLOSURETOP(i) vm->push(vm->doPop()->clone()->addClosureArgs(vm,i));
+#define CLOSURE(x,i,tot,lam) vm->push(MKPVAL(vm->getVPool(),(void*)(new Closure(vm,x,i,tot,lam)),KVT_FUNC))
+#define CLOSURELOC(x,i,lam) vm->push(x->clone()->addClosureArgs(vm,i,lam));
+#define CLOSURETOP(i,lam) vm->push(vm->doPop()->clone()->addClosureArgs(vm,i,lam));
 
 //#define GETFUN(x) (x->getFunc())
 #define GETVAL(t) (t=vm->doPop()->getInt())
@@ -90,7 +92,7 @@ class VMState;
 #define INTDIV(t,x,y) if (y==0) { vm->divideByZero(); } else { t=(x/y); }
 #define REALINFIX(t,op,x,y) t=(x op y)
 #define REALDIV(t,x,y) if (y==0) { vm->divideByZero(); } else { t=(x/y); }
-#define REALINFIXBOOL(op,x,y) vm->push(mkint((void*)(x op y)))
+#define REALINFIXBOOL(op,x,y) vm->push(mkint2(vm,(void*)(x op y)))
 // CIM: optimise infix jumps
 #define INFIXJFALSE(op,x,y,l) if ((x op y)==0) goto l
 #define INFIXJTRUE(op,x,y,l) if ((x op y)!=0) goto l
@@ -155,7 +157,7 @@ class VMState;
 #define STACKINT(x,i) stack_##x.setInt(i); vm->push(x);
 #define STACKREAL(x,r) stack_##x.setReal(r); vm->push(x);
 // doesn't quite work yet
-#define STACKSTR(x,s) stack_##x.setString(new String(s)); vm->push(x);
+#define STACKSTR(x,s) stack_##x.setString(new(vm->getSPool()) String(s)); vm->push(x);
 #define STACKRAW(x,i) stack_##x.setPtr((void*)(i), KVT_INT); vm->push(x);
 
 //#define SETVAL(x,y) x->setPtr(new Value((void*)y));
@@ -180,15 +182,15 @@ class VMState;
 #define PUSHGLOBAL(t,i) vm->push(KAYA_GLOBAL_##i)
 //#define PUSHGLOBAL(t,i) vm->pushglobal(t,i)
 #define CREATEGLOBAL(x,i) vm->newglobal(x,i)
-
-#define DECLGLOBAL(i) static Value* KAYA_GLOBAL_##i = MKVAL(0, KVT_INT);
+// definitely don't use MKPVAL here!
+#define DECLGLOBAL(i) static Value* KAYA_GLOBAL_##i = MKINITVAL(0, KVT_INT);
 
 #define TAG vm->tag()
-#define VMPTR vm->push(MKVAL((void*)vm,KVT_INT));
-#define STRLENGTH vm->push(MKVAL((void*)(vm->doPop()->getString()->length()),KVT_INT))
+#define VMPTR vm->push(MKPVAL(vm->getVPool(),(void*)vm,KVT_INT));
+#define STRLENGTH vm->push(MKPVAL(vm->getVPool(),(void*)(vm->doPop()->getString()->length()),KVT_INT))
 
-#define GETLENGTH vm->push(MKVAL((void*)(vm->topItem()->length()),KVT_INT))
-#define GETFNID vm->push(MKVAL((void*)(getFnID(vm->doPop()->getFunc())),KVT_INT))
+#define GETLENGTH vm->push(MKPVAL(vm->getVPool(),(void*)(vm->topItem()->length()),KVT_INT))
+#define GETFNID vm->push(MKPVAL(vm->getVPool(),(void*)(getFnID(vm->doPop()->getFunc())),KVT_INT))
 
 // Coercions
 #define STR2INT vm->str2int();
@@ -252,7 +254,9 @@ VMState* initstack();
 void finish(VMState* vm);
 
 Value* mkstr(const wchar_t* str);
+Value* mkstr2(VMState* vm,const wchar_t* str);
 Value* mkint(void* i);
+Value* mkint2(VMState* vm,void* i);
 Value* mkarrayval(void* i);
 
 /// Maths operators.
@@ -261,11 +265,16 @@ double realpower(double x, double y);
 
 /// Wide char/string conversions
 wchar_t* strtowc(const char* str);
+void strtovwc(VMState* vm, const char* str);
 wchar_t chrtowc(const char str);
 char* wctostr(const wchar_t* wc);
 char* wcntostr(const wchar_t* wc, kint len);
+// must manually ensure that char* target can hold (worst-case)
+// 4*len*sizeof(char) bytes when using wcnptostr
+
+void wcnptostr(const wchar_t* wc, kint len, char* target);
 char wctochr(const wchar_t* c);
 
-wchar_t* mkUCS(const char* utf8);
-void mkUTF8(const wchar_t* raw, char* c, unsigned int len);
+void mkUCS(const char* utf8, wchar_t* dest, kint inlen);
+void mkUTF8(const wchar_t* raw, char* c, ukint len);
 #endif
