@@ -41,7 +41,7 @@ using namespace std;
 #define INVALID_VALUE K_ns__D_Builtins__D_Invalid_UN_Value
 #define NOT_IMPLEMENTED K_ns__D_Builtins__D_Not_UN_Implemented
 #define INVALID_CIRCULAR_STRUCTURE K_ns__D_Builtins__D_Invalid_UN_Circular_UN_Structure
-#define UNKNOWN_FUNCTION_ID K_ns__D_Builtins__D_Unknown_UN_Function_UN_ID
+//#define UNKNOWN_FUNCTION_ID K_ns__D_Builtins__D_Unknown_UN_Function_UN_ID
 #define DIVIDE_BY_ZERO K_ns__D_Builtins__D_Divide_UN_By_UN_Zero
 
 extern "C" const char* PROJECT_FROM_NON_UNION;
@@ -56,7 +56,7 @@ extern "C" const char* NOT_IMPLEMENTED;
 extern "C" const char* INVALID_CIRCULAR_STRUCTURE;
 extern "C" const char* CANT_MARSHAL_EXCEPTIONS;
 extern "C" const char* CANT_REFLECT_EXCEPTIONS;
-extern "C" const char* UNKNOWN_FUNCTION_ID;
+//extern "C" const char* UNKNOWN_FUNCTION_ID;
 extern "C" const char* DIVIDE_BY_ZERO;
 
 /// Function map is global to all threads.
@@ -70,6 +70,7 @@ kint getFnHash();
 
 class Exception;
 
+
 class VMState: public gc {
     friend class Exception;
 public:
@@ -92,10 +93,12 @@ public:
         // single function will use and do a single check at the start
         // of the function?
         // In normal code the default stack size is enough anyway...
+
         if ((m_stackptr-m_valstack)>=m_stackalloc) {
 	  realloc_stack();
         }
 
+
 	*m_stackptr = val;
 	++m_stackptr;
     }
@@ -125,12 +128,13 @@ public:
     double topint2real();
 
     void mkArray(kint size);
+    void mkArray(Value* v, kint size);
     void finish();
     bool emptyStack();
 
 /// Replace the top stack item with its ith argument.
 /// Let's see if inlining helps
-    void projarg(unsigned i, unsigned t)
+    void projarg(ukint i, ukint t)
     {
 #ifndef NOCHECK
       Value*& topval = (*(m_stackptr-1));
@@ -142,7 +146,7 @@ public:
 	  kaya_rtsError(PROJECT_FROM_NON_UNION);
       }
       Union* top = (Union*)topval->getRaw();
-      if ((unsigned)(topval->getType()-KVT_UNION)!=t) {
+      if ((ukint)(topval->getType()-KVT_UNION)!=t) {
 	  kaya_rtsError(WRONG_CONSTRUCTOR);
       }
       if (U_ARITY(top) <= i) {
@@ -155,7 +159,7 @@ public:
       //    push(arg);
     }
 
-    void setprojarg(Value*& src, unsigned i, unsigned t, Value* dest)
+    void setprojarg(Value*& src, ukint i, ukint t, Value* dest)
     {
 #ifndef NOCHECK
       if (src==NULL) {
@@ -165,7 +169,7 @@ public:
 	  kaya_rtsError(PROJECT_FROM_NON_UNION);
       }
       Union* top = (Union*)src->getRaw();
-      if ((unsigned)(src->getType()-KVT_UNION)!=t) {
+      if ((ukint)(src->getType()-KVT_UNION)!=t) {
 	  kaya_rtsError(WRONG_CONSTRUCTOR);
       }
       if (U_ARITY(top) <= i) {
@@ -205,7 +209,7 @@ public:
 
     void setRVreal(double r)
     {
-	(*(m_stackptr-1))->setReal(r);
+	(*(m_stackptr-1))->setReal(m_rpool,r);
     }
 
 // Add the top stack value to the next one, then remove both.
@@ -249,8 +253,15 @@ public:
     {
 	// append top stack item onto top-1 stack item
 	wchar_t topstr = (wchar_t)((*(--m_stackptr))->getInt());
+	// and now adjust the stack to correct for an explicit coercion
+	// being removed.
+	// FIXME: probably a better way to do this than a peephole optimisation
+	// anyway!
+	discardPop();
+	// add the right values
 	(*(m_stackptr-1))->getString()->append(topstr);
 	*m_stackptr = NULL;
+
     }
 
     /// Return the top stack item.
@@ -340,6 +351,33 @@ public:
     void realloc_stack();
     void realloc_callstack();
 
+    ValuePool* getVPool() {
+        return m_vpool;
+    }
+    UnionPool* getUPool() {
+        return m_upool;
+    }
+    StringPool* getSPool() {
+        return m_spool;
+    }
+    RealPool* getRPool() {
+        return m_rpool;
+    }
+    ArrayPool* getAPool() {
+        return m_apool;
+    }
+
+  // Temporary string buffer to reduce allocations needed in string handling
+  // If this works well then it should be done in a neater way!
+    wchar_t* allocString(kint alloc) {
+      strbuffer = (wchar_t*)KMALLOC_BLOCK(alloc*sizeof(wchar_t));
+      strbuflen = alloc;
+      strbuffer[0] = '\0';
+      return strbuffer;
+    }
+    wchar_t* strbuffer;
+    kint strbuflen;
+
 private:
     Value** m_valstack;
     Value** m_stackptr;
@@ -347,6 +385,12 @@ private:
 
 //    int m_stacksize;
 
+    ValuePool* m_vpool;
+    UnionPool* m_upool;
+    StringPool* m_spool;
+    RealPool* m_rpool;
+    ArrayPool* m_apool;
+
     int m_stackalloc;
     int m_callstackalloc;
     int m_globalloc;
@@ -367,6 +411,7 @@ private:
     int m_maxmem;
 
     Value m_return; // Place to put return values (sort of a register).
+
 };
 
 #endif // whole file
