@@ -25,6 +25,10 @@ kint inttable_fasteq(Value* x, Value* y)
     return (x->getInt()==y->getInt());
 }
 
+kint inttable_hash(Value* x) {
+  return x->getInt();
+}
+
 kint inttable_cmp(Value* x, Value* y)
 {
     int xi = x->getInt();
@@ -34,13 +38,9 @@ kint inttable_cmp(Value* x, Value* y)
     else return 1;
 }
 
-Value* inttable_copy(Value* x, map<Value*,Value*>& done)
+Value* inttable_fastcopy(VMState* vm, Value* x)
 {
-    return MKVAL((void*)(x->getInt()),KVT_INT);
-}
-Value* inttable_fastcopy(Value* x)
-{
-    return MKVAL((void*)(x->getInt()),KVT_INT);
+    return MKPVAL(vm->getVPool(),(void*)(x->getInt()),KVT_INT);
 }
 
 void inttable_marshal(VMState* vm,String* mdata,vector<Value*>& done,Value* x, kint id)
@@ -85,11 +85,11 @@ kint stringtable_cmp(Value* x, Value* y)
     else return 0;
 }
 
-Value* stringtable_copy(Value* x, map<Value*,Value*>& done)
-{
-    return MKSTR(x->getString()->getVal());
+kint stringtable_hash(Value* x) {
+  return strHash(x->getString()->getVal(),x->getString()->length());
 }
-Value* stringtable_fastcopy(Value* x)
+
+Value* stringtable_fastcopy(VMState* vm, Value* x)
 {
     return MKSTR(x->getString()->getVal());
 }
@@ -140,9 +140,28 @@ kint fntable_cmp(Value* x, Value* y)
     return (kint)(x->getRaw())-(kint)(y->getRaw());
 }
 
-Value* fntable_copy(Value* x, map<Value*,Value*>& done)
+Value* fntable_copy(VMState* vm, Value* x, map<Value*,Value*>& done)
 {
-    return MKVAL((void*)(x->getFunc()->copy(done)),KVT_FUNC);
+    return MKPVAL(vm->getVPool(),(void*)(x->getFunc()->copy(vm,done)),KVT_FUNC);
+}
+
+kint fntable_hash(Value* x,vector<Value*>& done) {
+    int idx;
+    vector<Value*>::iterator it = done.begin();
+    for(idx=0;it!=done.end();++it,++idx) {
+	if ((*it)->getRaw()==x->getRaw()) {
+	    return 0;
+	}
+    }
+    idx = done.size();
+    done.push_back(x);
+    kint hash = 1;
+    Closure *c = x->getFunc();
+    Value** cargs = c->getArgs();
+    for(int i=0;i<c->getNumArgs();++i) {
+	hash += funtable_hash_aux(cargs[i],done);
+    }
+    return hash;
 }
 
 void fntable_marshal(VMState* vm,String* mdata,vector<Value*>& done,Value* x, kint id)
@@ -244,18 +263,11 @@ kint realtable_cmp(Value* x, Value* y)
     else return 1;
 }
 
-Value* realtable_copy(Value* x, map<Value*,Value*>& done)
+Value* realtable_fastcopy(VMState* vm, Value* x)
 {
     double xv = x->getReal();
-    Value* v=MKVAL(NULL,KVT_REAL);
-    v->setReal(xv);
-    return v;
-}
-Value* realtable_fastcopy(Value* x)
-{
-    double xv = x->getReal();
-    Value* v=MKVAL(NULL,KVT_REAL);
-    v->setReal(xv);
+    Value* v=MKPVAL(vm->getVPool(),NULL,KVT_REAL);
+    v->setReal(vm->getRPool(),xv);
     return v;
 }
 
@@ -274,6 +286,10 @@ Value* realtable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
     return v;
 }
 
+kint realtable_hash(Value* x)
+{
+  return (kint)x->getReal();
+}
 int realtable_memusage(Value* x)
 {
     return sizeof(Value)+sizeof(Real);
@@ -301,17 +317,35 @@ kint arraytable_cmp(Value* x, Value* y)
     else return 0;
 }
 
-Value* arraytable_copy(Value* x, map<Value*,Value*>& done)
+Value* arraytable_copy(VMState* vm, Value* x, map<Value*,Value*>& done)
 {
     // if x is already copied, return the copied value
     if (done.find(x)!=done.end()) {
 	return done[x];
     }
-    Value* copied = MKVAL(x->getArray()->copy(done),KVT_ARRAY);
+    Value* copied = MKPVAL(vm->getVPool(),x->getArray()->copy(vm,done),KVT_ARRAY);
     done[x]=copied;
     return copied;
 }
 
+kint arraytable_hash(Value* x, vector<KayaValue>& done) {
+    int idx;
+    vector<Value*>::iterator it = done.begin();
+    for(idx=0;it!=done.end();++it,++idx) {
+	if ((*it)->getRaw()==x->getRaw()) {
+	    return 0;
+	}
+    }
+    idx = done.size();
+    done.push_back(x);
+    kint hash = 1;
+    Array* v = x->getArray();
+    for(ukint i=0;i<v->size();++i) {
+	hash += funtable_hash_aux(v->lookup(i),done);
+    }
+    return hash;
+}
+
 void arraytable_marshal(VMState* vm,String* mdata,vector<KayaValue>& done,Value* x, kint id)
 {
     // Check for circles.
@@ -333,7 +367,7 @@ void arraytable_marshal(VMState* vm,String* mdata,vector<KayaValue>& done,Value*
     wchar_t buf[34];
     SWPRINTF(buf,34,ARRMARSHAL,idx,v->reserved_size());
     mdata->append(buf);
-    for(unsigned i=0;i<v->size();++i) {
+    for(ukint i=0;i<v->size();++i) {
 	funtable_marshal_aux(vm,mdata,done,v->lookup(i),id);
     }
     mdata->append(L"]");
@@ -352,7 +386,7 @@ Value* arraytable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
 
     KArray ar = newKayaArray(inarr->size());
 
-    for(unsigned i=0;i<inarr->size();++i) {
+    for(ukint i=0;i<inarr->size();++i) {
 	Value* el = funtable_reflect_aux(vm,done,inarr->lookup(i));
 	KayaArrayPush(ar,el);
     }
@@ -405,15 +439,35 @@ kint uniontable_cmp(Value* x, Value* y)
   }
 }
 
-Value* uniontable_copy(Value* x, map<Value*,Value*>& done)
+Value* uniontable_copy(VMState* vm,Value* x, map<Value*,Value*>& done)
 {
-    Value* thisval = MKVAL(NULL,KVT_NULL);
+    Value* thisval = MKPVAL(vm->getVPool(),NULL,KVT_NULL);
     done[x] = thisval;
     //    Value* copied = MKVAL(,x->getType());
-    thisval->setPtr(x->getUnion()->copy(done),x->getType()); 
+    thisval->setPtr(x->getUnion()->copy(vm,done),x->getType()); 
     return thisval;
 }
 
+kint uniontable_hash(Value* x, vector<KayaValue>& done) {
+    int idx;
+    vector<Value*>::iterator it = done.begin();
+    for(idx=0;it!=done.end();++it,++idx) {
+	if ((*it)->getRaw()==x->getRaw()) {
+	    return 0;
+	}
+    }
+    idx = done.size();
+    done.push_back(x);
+    
+    kint hash = 1;
+    Union *u = x->getUnion();
+    hash += x->getType();
+    for(ukint i=0;i<U_ARITY(u);++i) {
+	hash+=funtable_hash_aux(u->args[i],done);
+    }
+    return hash;
+}
+
 void uniontable_marshal(VMState* vm,String* mdata,vector<KayaValue>& done,Value* x, kint id)
 {
     // Check for circles.
@@ -439,7 +493,7 @@ void uniontable_marshal(VMState* vm,String* mdata,vector<KayaValue>& done,Value*
     SWPRINTF(utag,50,MARSHAL2FORMAT,x->getType()-KVT_UNION,U_ARITY(u));
     mdata->append(utag);
 
-    for(unsigned i=0;i<U_ARITY(u);++i) {
+    for(ukint i=0;i<U_ARITY(u);++i) {
 	funtable_marshal_aux(vm,mdata,done,u->args[i],id);
     }
     mdata->append(L"]");
@@ -460,7 +514,7 @@ Value* uniontable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
 
     KArray flds = newKayaArray(U_ARITY(inu));
 
-    for(unsigned i=0;i<U_ARITY(inu);++i) {
+    for(ukint i=0;i<U_ARITY(inu);++i) {
 	Value* el = funtable_reflect_aux(vm,done,inu->args[i]);
 	KayaArrayPush(flds,el);
     }
@@ -495,9 +549,9 @@ kint cuniontable_cmp(Value* x, Value* y)
     else return 1;
 }
 
-Value* cuniontable_copy(Value* x, map<Value*,Value*>& done)
+Value* cuniontable_copy(VMState* vm, Value* x, map<Value*,Value*>& done)
 {
-    Value* thisval = MKVAL((void*)x->getInt(),KVT_CUNION);
+    Value* thisval = MKPVAL(vm->getVPool(),(void*)x->getInt(),KVT_CUNION);
     return thisval;
 }
 
@@ -546,6 +600,10 @@ Value* cuniontable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
     return v;
 }
 
+kint cuniontable_hash(Value* x) {
+  return x->getInt();
+}
+
 int cuniontable_memusage(Value* x)
 {
     int size = sizeof(Value);
@@ -555,6 +613,10 @@ int cuniontable_memusage(Value* x)
 
 /// Functions on Exceptions
 
+kint exceptiontable_hash(Value* x) {
+  return 0;
+}
+
 kint exceptiontable_eq(Value* x, Value* y, map<Value*, Value*>& done)
 {
     return (x->getExcept()->eq(y->getExcept()));
