@@ -85,14 +85,15 @@ kint strtoint(wchar_t* str, kint base)
 #endif
 }
 
-wchar_t* getsubstr(wchar_t* x,kint i,kint len)
+VMState* getsubstr(void* vmptr, wchar_t* x,kint i,kint len)
 {
-    wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((len+1)*sizeof(wchar_t));
+    VMState* vm = (VMState*)vmptr;
+    wchar_t* n = vm->allocString(len<<1);
     if (len > 0) {
         wcsncpy(n,x+i,len);
     }
     n[len] = '\0';
-    return n;
+    return vm;
 }
 
 void str_offset(Value* str, kint inc)
@@ -105,15 +106,16 @@ void str_chop(Value* str, kint inc)
     str->getString()->chop(inc);
 }
 
-wchar_t* getstrend(wchar_t* x,kint i)
+VMState* getstrend(void* vmptr, wchar_t* x,kint i)
 {
     kint max = wcslen(x);
-    wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((max+1-i)*sizeof(wchar_t));
+    VMState* vm = (VMState*)vmptr;
+    wchar_t* n = vm->allocString(max+1-i);
     if (max-i > 0) {
       wcsncpy(n,x+i,max-i);
     }
     n[max-i] = '\0';
-    return n;
+    return vm;
 }
 wchar_t getIndex(wchar_t* str,kint i)
 {
@@ -125,12 +127,14 @@ void setIndex(wchar_t* str, wchar_t c,kint i)
     str[i] = c;
 }
 
-wchar_t* getLine(void* fptr)
+void getLineTo(void* fptr,KayaValue strv)
 {
+    String* str = strv->getString();
     FILE* f = (FILE*)fptr;
     unsigned size = LINE_CHUNK; // #define at top of file
-    // buffer to read into
-    char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
+    // buffer to read into (since it's not going anywhere near the
+    // Kaya VM except as a stack variable, no need to allocate)
+    char buf[LINE_CHUNK+1];
     // accumulator for result; but only allocate it when it turns out to be
     // needed (i.e., when one chunk isn't enough).
     char* res=NULL;
@@ -141,10 +145,16 @@ wchar_t* getLine(void* fptr)
 	    if (res==NULL)
 	    {
 		// Fail silently (should this throw an exception?)
-		return emptyString(); 
+	      //	      free(buf);
+	      str->getForeign("");
+	      //	      vm->allocString(1);
+	      return; 
 	    } else {
 		// File finished without a newline, return what we've got.
-		return strtowc(res);
+	      str->getForeign(res);
+		free(res);
+		//		free(buf);
+		return;
 	    }
 	}
 	if (res==NULL) {
@@ -153,26 +163,109 @@ wchar_t* getLine(void* fptr)
 	    // C defines \n to be the native newline when reading
 	    // or writing files in text mode so maybe we're okay...
 	    if (strlen(buf)<(size-1) || buf[LINE_CHUNK-2]=='\n') {
-		return strtowc(buf); // Success in one chunk
+	      str->getForeign(buf);
+	      //KVSTRING(vm,buf); // Success in one chunk
+		//		free(buf);
+		return;
 	    }
 	    // Begin allocation for accumulator and carry on as normal.
 	    size += LINE_CHUNK-1;
-	    res = (char*)GC_MALLOC_ATOMIC(size*sizeof(char)+1);
+	    res = (char*)malloc(size*sizeof(char)+1);
 	    strcpy(res,buf);
 	} 
 	else {
 	    strcat(res,buf);
 	    if (strlen(res)<(size-1) || buf[LINE_CHUNK-2]=='\n') { 
                 // Finished successfully
-		return strtowc(res); 
+	      str->getForeign(res);
+	      //		KVSTRING(vm,res); 
+		free(res);
+		//		free(buf);
+		return;
 	    } else {
 		size += LINE_CHUNK-1; // -1 to account for null termination of
 		                      // string read into buf
-		res = (char*)GC_REALLOC(res, size*sizeof(char)+1);
+		res = (char*)realloc(res, size*sizeof(char)+1);
 	    }
 	}
     } while (r!=NULL);
-    return emptyString(); // Failed, although we shouldn't get here
+    //    free(buf);
+    if (res != NULL) {
+      free(res);
+    }
+    //    vm->allocString(1);
+    str->getForeign("");
+    //    return vm; 
+    // Failed, although we shouldn't get here
+}
+
+
+VMState* getLine(void* vmptr,void* fptr)
+{
+    VMState* vm = (VMState*)vmptr;
+    FILE* f = (FILE*)fptr;
+    unsigned size = LINE_CHUNK; // #define at top of file
+    // buffer to read into (since it's not going anywhere near the
+    // Kaya VM except as a stack variable, no need to allocate)
+    char buf[LINE_CHUNK+1];
+    // accumulator for result; but only allocate it when it turns out to be
+    // needed (i.e., when one chunk isn't enough).
+    char* res=NULL;
+    char* r;
+    do {
+	r = fgets(buf,LINE_CHUNK,f);
+	if (r == NULL) { // End of file, or read error. FIXME: Check which.
+	    if (res==NULL)
+	    {
+		// Fail silently (should this throw an exception?)
+	      //	      free(buf);
+	      vm->allocString(1);
+	      return vm; 
+	    } else {
+		// File finished without a newline, return what we've got.
+		KVSTRING(vm,res);
+		free(res);
+		//		free(buf);
+		return vm;
+	    }
+	}
+	if (res==NULL) {
+	    // FIXME: We're relying on \n or \r\n line endings. How much
+	    // can we rely on this?
+	    // C defines \n to be the native newline when reading
+	    // or writing files in text mode so maybe we're okay...
+	    if (strlen(buf)<(size-1) || buf[LINE_CHUNK-2]=='\n') {
+		KVSTRING(vm,buf); // Success in one chunk
+		//		free(buf);
+		return vm;
+	    }
+	    // Begin allocation for accumulator and carry on as normal.
+	    size += LINE_CHUNK-1;
+	    res = (char*)malloc(size*sizeof(char)+1);
+	    strcpy(res,buf);
+	} 
+	else {
+	    strcat(res,buf);
+	    if (strlen(res)<(size-1) || buf[LINE_CHUNK-2]=='\n') { 
+                // Finished successfully
+		KVSTRING(vm,res); 
+		free(res);
+		//		free(buf);
+		return vm;
+	    } else {
+		size += LINE_CHUNK-1; // -1 to account for null termination of
+		                      // string read into buf
+		res = (char*)realloc(res, size*sizeof(char)+1);
+	    }
+	}
+    } while (r!=NULL);
+    //    free(buf);
+    if (res != NULL) {
+      free(res);
+    }
+    vm->allocString(1);
+    return vm; 
+    // Failed, although we shouldn't get here
 }
 
 wchar_t do_fgetc(void* fptr) {
@@ -187,12 +280,14 @@ void* do_tmpFile() {
   return (void*)tmpfile();
 }
 
-wchar_t* getString(void* fptr)
+VMState* getString(void* vmptr, void* fptr)
 {
+    VMState* vm = (VMState*)vmptr;
     FILE* f = (FILE*)fptr;
     unsigned size = LINE_CHUNK; // #define at top of file
-    // buffer to read into
-    char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
+    // buffer to read into (since it's not going anywhere near the
+    // Kaya VM itself, use malloc rather than KMALLOC and free before exit)
+    char* buf=(char*)malloc(LINE_CHUNK*sizeof(char)+1); 
     // accumulator for result; but only allocate it when it turns out to be
     // needed (i.e., when one chunk isn't enough).
     int i=0;
@@ -202,31 +297,40 @@ wchar_t* getString(void* fptr)
 	int c = fgetc(f);
 	if (c == EOF) {
 	  buf[i] = '\0';
-	  return strtowc(buf);
+	  KVSTRING(vm,buf);
+	  free(buf);
+	  return vm;
 	} 
 	buf[i] = (unsigned char)c;
 	if (buf[i] == '\0') {
-	  return strtowc(buf);
+	  KVSTRING(vm,buf);
+	  free(buf);
+	  return vm;
 	}
 	i++;
       } while (i%LINE_CHUNK != 0);
       size += LINE_CHUNK;
-      buf = (char*)GC_REALLOC(buf, size*sizeof(char)+1);
+      buf = (char*)realloc(buf, (size+1)*sizeof(char));
 
-    } while (buf[i] != EOF);
-    return emptyString(); // Failed, although we shouldn't get here
+    } while (true); // replaces != EOF because the check above will get that
+    // ...and unsigned char can never equal EOF anyway.
+    free(buf);
+    vm->allocString(1);
+    return vm; // Failed, although we can't get here
 }
 
 void putLine(void* fptr, wchar_t* c, kint len)
 {
     FILE* f = (FILE*)fptr;
-    fputs(wcntostr(c,len),f);
+    CNPSTRING(r,c,len);
+    fputs(r,f);
 }
 
 void putString(void* fptr, wchar_t* c, kint len)
 {
     FILE* f = (FILE*)fptr;
-    fputs(wcntostr(c,len),f);
+    CNPSTRING(r,c,len);
+    fputs(r,f);
     fputc('\0',f);
 }
 
@@ -284,7 +388,7 @@ kint checkUTF8(wchar_t* str)
 
 void printArray(KayaArray foo)
 {
-    for(unsigned i=0;i<foo->size();++i) {
+    for(ukint i=0;i<foo->size();++i) {
 	cout << wctostr(foo->lookup(i)->getString()->getVal()) << endl;
     }
 }
@@ -292,7 +396,7 @@ void printArray(KayaArray foo)
 KayaArray reverseArray(KayaArray foo)
 {
     KayaArray bar = new Array();
-    for(unsigned i=foo->size();i>0;i--) {
+    for(ukint i=foo->size();i>0;i--) {
 	bar->push_back(foo->lookup(i-1));
     }
     return bar;
@@ -315,9 +419,9 @@ void unshiftArray(KayaValue val, KayaArray foo)
     return foo->unshift(val);
 }
 
-KayaArray createArray(kint size)
+KayaArray createArray(void* vmptr, kint size)
 {
-    return newKayaArray(size);
+    return newKayaArray2(vmptr,size);
 }
 
 void resizeArray(KayaArray foo, kint size)
@@ -383,8 +487,9 @@ kint dogettimeofday() {
   return tv.tv_usec;
 }
 
-#ifdef WIN32
+#ifdef GTOD_INTERNAL
 // taken from libiberty (LGPL) in the GCC libraries
+// needed for old GCC versions on Windows
 kint gettimeofday (struct timeval *tp, void *tz)
 {     
   /* Offset between 1/1/1601 and 1/1/1970 in 100 nanosec units */
@@ -467,8 +572,8 @@ kint domktime(KayaValue time)
     t.tm_year = (tu->args[5]->getInt())-1900;
     t.tm_yday = tu->args[7]->getInt();
     t.tm_isdst = tu->args[8]->getInt();
-    t.tm_mon = tu->args[4]->getType();
-    t.tm_wday = tu->args[6]->getType();
+    t.tm_mon = KayaUnionGetTag(tu->args[4]);
+    t.tm_wday = KayaUnionGetTag(tu->args[6]);
 
     kint x = (kint)mktime(&t);
     return x;
@@ -484,7 +589,7 @@ kint runProgram(wchar_t* prog, KayaArray args)
     char** argv = new char*[args->size()+2];
 
     argv[0]=wctostr(prog);
-    for(unsigned i=0;i<args->size();++i) {
+    for(ukint i=0;i<args->size();++i) {
 	argv[i+1]=wctostr(args->lookup(i)->getString()->getVal());
     }
     argv[args->size()+1]=NULL;
@@ -574,7 +679,8 @@ kint offset_wcscspn(wchar_t c, wchar_t* s, kint i) {
  * By Bob Trower, Copyright (c) Trantor Standard Systems Inc., 2001
  * See COPYING.b64 for redistribution conditions
  */
-// Modified by CIM to work with wchar_t input
+// Modified by CIM to work with wchar_t input and for other
+// integration with Kaya
 /*
 ** Translation Table as described in RFC1113
 */
@@ -886,11 +992,53 @@ kint funtable_memusage(KayaValue x)
     return r;
 }
 
-wchar_t* funtable_marshal(void* vmptr,KayaValue x, kint i)
+kint funtable_hash(KayaValue x) {
+  vector<KayaValue> done;
+  return funtable_hash_aux(x,done);
+}
+
+kint funtable_hash_aux(KayaValue x,vector<KayaValue>& done)
 {
+    kint r = 0;
+    switch (x->getFunTable()) { 
+    case KVT_INT:
+        r = inttable_hash(x); 
+	break; 
+    case KVT_STRING: 
+	r = stringtable_hash(x); 
+	break; 
+    case KVT_FUNC: 
+	r = fntable_hash(x,done); 
+	break; 
+    case KVT_ARRAY: 
+	r = arraytable_hash(x,done); 
+	break; 
+    case KVT_CUNION: 
+	r = cuniontable_hash(x); 
+	break; 
+    case KVT_EXCEPTION: 
+	r = exceptiontable_hash(x); 
+	break; 
+    case KVT_REAL: 
+	r = realtable_hash(x); 
+	break; 
+    case KVT_NULL: 
+	r = inttable_hash(x);
+	break;
+    case KVT_UNION: 
+    default:
+	r = uniontable_hash(x,done); 
+	break; 
+    } 
+    return r;
+}
+
+
+VMState* funtable_marshal(void* vmptr,KayaValue x, kint i)
+{
+  VMState* vm = (VMState*)vmptr;
 #ifndef NOCHECK
     if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL)) {
-	VMState* vm = (VMState*)vmptr;
 	vm->kaya_rtsError(INVALID_VALUE);
     }
 #endif
@@ -898,9 +1046,13 @@ wchar_t* funtable_marshal(void* vmptr,KayaValue x, kint i)
     
     wchar_t mid[50]; // can be negative
     SWPRINTF(mid,50,MARSHAL2FORMAT,i,getFnHash());
-    String* idmdata = NEWSTRING(mid);
+    String* idmdata = NEWPSTRING(vm->getSPool(),mid);
     funtable_marshal_aux(vmptr,idmdata,done,x,i);
-    return idmdata->getVal();
+    vm->strbuffer = idmdata->getVal();
+    // FIXME: this isn't strictly right, but as long as m_offset in
+    // the String is 0 which it will be for this String, it's correct.
+    vm->strbuflen = idmdata->space();
+    return vm;
 }
 
 void funtable_marshal_aux(void* vmptr,String* mdata,vector<KayaValue>& done,KayaValue x, kint i)
@@ -995,7 +1147,7 @@ KayaValue reifyUnion(kint tag, kint arity)
 
 void reifyUnionArgs(KayaValue v, KayaArray flds)
 {
-    for(unsigned i=0;i<flds->size();++i) {
+    for(ukint i=0;i<flds->size();++i) {
 	KayaUnionSetArg(v,i,KayaArrayLookup(flds,i));
     }
 }
@@ -1009,39 +1161,68 @@ KayaValue reifyClosure(kint fnid, kint arity)
 void reifyClosureArgs(KayaValue v, KayaArray flds)
 {
     Closure* c = v->getFunc();
-    for(unsigned i=0;i<flds->size();++i) {
+    for(ukint i=0;i<flds->size();++i) {
 	c->setArg(i,KayaArrayLookup(flds,i));
     }
 }
 
 KayaValue funtable_copy(void* vmptr,KayaValue x)
 {
+  VMState* vm = (VMState*)vmptr;
 #ifndef NOCHECK
   if (x == NULL || x->getType() == KVT_NULL) {
-    VMState* vm = (VMState*)vmptr;
     vm->kaya_rtsError(INVALID_VALUE);
   }
 #endif
     valtype vt = x->getType();
     switch (vt) {
     case KVT_INT:
-      return inttable_fastcopy(x);
+      return inttable_fastcopy(vm,x);
     case KVT_STRING:
-      return stringtable_fastcopy(x);
+      return stringtable_fastcopy(vm,x);
     case KVT_REAL:
-      return realtable_fastcopy(x);
+      return realtable_fastcopy(vm,x);
     case KVT_EXCEPTION:
       return exceptiontable_fastcopy(x);
     default:
       map<Value*, Value*> done;
-      return funtable_copy_aux(x,done);
+      return funtable_copy_aux(vm,x,done);
     }
 }
 
-KayaValue funtable_copy_aux(KayaValue x, map<Value*, Value*>& done)
+KayaValue funtable_copy_aux(VMState* vm, KayaValue x, map<Value*, Value*>& done)
 {
     KayaValue r; // = MKVAL(NULL,KVT_NULL);
-    FUNTABLE_OP(copy,x,done)
+    switch (x->getFunTable()) { 
+    case KVT_INT: 
+      r = inttable_fastcopy(vm,x);
+      break;
+    case KVT_STRING: 
+      r = stringtable_fastcopy(vm,x);
+      break;
+    case KVT_FUNC: 
+      r = fntable_copy(vm,x,done);
+      break;
+    case KVT_ARRAY: 
+      r = arraytable_copy(vm,x,done); 
+      break;
+    case KVT_CUNION: 
+      r = cuniontable_copy(vm,x,done);
+      break;
+    case KVT_EXCEPTION: 
+      r = exceptiontable_fastcopy(x);
+      break;
+    case KVT_REAL: 
+      r = realtable_fastcopy(vm,x); 
+      break;
+    case KVT_NULL:
+      r = inttable_fastcopy(vm,x);
+      break; //can't happen
+    case KVT_UNION: 
+    default:
+      r = uniontable_copy(vm,x,done);
+      break;
+    } 
     return r;
 }
 
@@ -1084,7 +1265,7 @@ kint doGetFnID(KayaValue fn)
     return (kint)getFnID(fn->getFunc()->getfn());
 }
 
-void callFnID(void* vmptr, kint id, KayaValue arg)
+/*void callFnID(void* vmptr, kint id, KayaValue arg)
 {
     VMState* vm = (VMState*)vmptr;
     func f = getFn(id);
@@ -1094,7 +1275,8 @@ void callFnID(void* vmptr, kint id, KayaValue arg)
     } else {
 	vm->kaya_rtsError(UNKNOWN_FUNCTION_ID);
     }
-}
+}*/
+
 #ifndef WIN32
 kint memusage()
 {
@@ -1272,10 +1454,12 @@ void* do_readdir(void* dir)
     return (void*)ent;
 }
 
-wchar_t* dir_getname(void* ent)
+VMState* dir_getname(void* vmptr, void* ent)
 {
+    VMState* vm = (VMState*)vmptr;
     struct dirent* entd = (struct dirent*)ent;
-    return strtowc(entd->d_name);
+    KVSTRING(vm,entd->d_name);
+    return vm;
 }
 
 #define SSET(x,y) KayaUnionSetArg(val,x,KayaInt((int)(buf->y)))
@@ -1385,16 +1569,21 @@ kint maxMemUsage(void* vmptr)
     return (kint)vm->maxMemUsage();
 }
 
-wchar_t* do_urlencode(void* vm, wchar_t* strinwide, kint len)
+VMState* do_urlencode(void* vm, wchar_t* strinwide, kint len)
 {
-  char* strin = CNSTRING(strinwide,len);
+  //    char strin[( len +1)*4];
+  //    wcnptostr(strin,strinwide,len);
+    CNPSTRING(si,strinwide,len);
+    char* strin = si;
     VMState* vmptr = (VMState*)vm;
-    char* strout = (char*)KayaAlloc(strlen(strin)*3+1);
-    if (strout==NULL) {
+    int maxlen = strlen(strin)*3+1;
+    char s[maxlen];
+    if (s==NULL) {
 	// something broke, probably duff input.
 	vmptr->kaya_internalError(1);
     }
-    char* r = strout;
+    char* r = s;
+    char* strout = s;
     while(*strin!='\0') {
 	if (*strin==' ') {
 	    *strout = '+';
@@ -1414,14 +1603,19 @@ wchar_t* do_urlencode(void* vm, wchar_t* strinwide, kint len)
 	++strin;
     }
     *strout='\0';
-    return KSTRING(r);
+    KVSTRING(vmptr,r);
+    return vmptr;
 }
 
-wchar_t* do_urldecode(void* vm, wchar_t* strinwide, kint len)
+VMState* do_urldecode(void* vm, wchar_t* strinwide, kint len)
 {
-  char* strin = CNSTRING(strinwide,len);
+    CNPSTRING(si,strinwide,len);
+    char* strin = si;
+
     VMState* vmptr = (VMState*)vm;
-    char* strout = (char*)KayaAlloc(strlen(strin)+1);
+    int maxlen = strlen(strin)+1;
+    char s[maxlen];
+    char* strout = s;
     if (strout==NULL) {
 	// something broke, probably duff input.
 	vmptr->kaya_internalError(1);
@@ -1457,7 +1651,8 @@ wchar_t* do_urldecode(void* vm, wchar_t* strinwide, kint len)
 	++strin;
     }
     *strout='\0';
-    return KSTRING(r);
+    KVSTRING(vmptr,r);
+    return vmptr;
 }
 
 void* do_fopen(const wchar_t* path, const wchar_t* mode) {
@@ -1500,7 +1695,7 @@ kint do_pclose(void* stream) {
 #endif
 
 kint binary_length(const wchar_t* str, kint len) {
-    char* rawstr = CNSTRING(str,len);
+    CNPSTRING(rawstr,str,len);
     return (kint)strlen(rawstr);
 }
 
@@ -1518,9 +1713,10 @@ kint strAlloc(Value* str)
     return str->getString()->space()*sizeof(wchar_t);
 }
 
-Value* createString(kint len)
+Value* createString(void* vmptr,kint len)
 {
-    return MKVAL((void*)(NEWSTRING(len)),KVT_STRING);
+  VMState* vm = (VMState*) vmptr;
+    return MKPVAL(vm->getVPool(),(void*)(NEWPSTRING(vm->getSPool(),len)),KVT_STRING);
 }
 
 void do_GC_enable_incremental() {
@@ -1529,6 +1725,14 @@ void do_GC_enable_incremental() {
 #endif
 }
 
+void runFn(void* obj, void* finalizer) {
+    KayaCall((Value*)finalizer, 1, (Value*)obj);
+}
+
+void do_GC_finalizer(Value* obj, Value* finalizer) {
+    GC_register_finalizer(obj, runFn, finalizer, NULL, NULL);
+}
+
 void do_sleep(kint len) {
 #ifdef WIN32
 	Sleep(len*1000);
