@@ -28,24 +28,25 @@ extern KayaArray args;
 wchar_t* inttostr(kint s, kint base);
 kint strtoint(wchar_t* str, kint base);
 
-wchar_t* getsubstr(wchar_t* x,kint i,kint len);
+VMState* getsubstr(void* vmptr, wchar_t* x,kint i,kint len);
 void str_offset(Value* str, kint inc);
 void str_chop(Value* str, kint inc);
-wchar_t* getstrend(wchar_t* x,kint i);
+VMState* getstrend(void* vmptr, wchar_t* x,kint i);
 wchar_t* readstr();
 
 wchar_t getIndex(wchar_t* str,kint i);
 void setIndex(wchar_t* str, wchar_t c,kint i);
 
-wchar_t* do_urlencode(void* vm, wchar_t* strinwide, kint len);
-wchar_t* do_urldecode(void* vm, wchar_t* strinwide, kint len);
+VMState* do_urlencode(void* vm, wchar_t* strinwide, kint len);
+VMState* do_urldecode(void* vm, wchar_t* strinwide, kint len);
 
 void* do_tmpFile();
 
-wchar_t* getLine(void* file);
+VMState* getLine(void* vmptr, void* file);
+void getLineTo(void* file, KayaValue strv);
 wchar_t do_fgetc(void* fptr);
 void do_fputc(kint ch, void* fptr);
-wchar_t* getString(void* file);
+VMState* getString(void* vmptr, void* file);
 void putString(void* f, wchar_t* c, kint len);
 void putLine(void* f, wchar_t* c, kint len);
 void do_fseek(void* f, kint p);
@@ -71,7 +72,7 @@ KayaArray reverseArray(KayaArray foo);
 void shortenArray(KayaArray foo);
 KayaValue shiftArray(KayaArray foo);
 void unshiftArray(KayaValue val, KayaArray foo);
-KayaArray createArray(kint size);
+KayaArray createArray(void* vmptr, kint size);
 void resizeArray(KayaArray foo, kint size);
 kint arraySize(KayaArray foo);
 
@@ -82,7 +83,7 @@ KayaArray getArgs();
 kint gettime();
 kint getclock();
 kint dogettimeofday();
-#ifdef WIN32
+#ifdef GTOD_INTERNAL
 kint gettimeofday (struct timeval *tp, void *tz);
 #endif
 KayaValue dogmtime(kint secs);
@@ -154,13 +155,15 @@ kint funtable_compare(void* vmptr, KayaValue x, KayaValue y);
 bool funtable_equal(void* vmptr, KayaValue x, KayaValue y);
 bool funtable_equal_aux(KayaValue x, KayaValue y, map<Value*,Value*>& done);
 kint funtable_memusage(KayaValue x);
-wchar_t* funtable_marshal(void* vmptr,KayaValue x,kint id);
+kint funtable_hash(KayaValue x);
+kint funtable_hash_aux(KayaValue x, vector<KayaValue>& done);
+VMState* funtable_marshal(void* vmptr,KayaValue x,kint id);
 void funtable_marshal_aux(void* vmptr,String* mdata,vector<KayaValue>& done,KayaValue x, kint i);
 Value* funtable_reflect(void* vmptr,KayaValue x);
 Value* funtable_reflect_aux(void* vmptr,
 			    map<KayaValue, KayaValue>& done,KayaValue x);
 KayaValue funtable_copy(void* vmptr, KayaValue x);
-KayaValue funtable_copy_aux(KayaValue x, map<Value*, Value*>& done);
+KayaValue funtable_copy_aux(VMState* vm, KayaValue x, map<Value*, Value*>& done);
 
 KayaValue reifyUnion(kint tag, kint arity);
 void reifyUnionArgs(KayaValue v, KayaArray flds);
@@ -177,7 +180,7 @@ kint except_code(KayaValue x);
 void except_dumpbt(KayaValue x);
 
 kint doGetFnID(KayaValue fn);
-void callFnID(void* vmptr, kint id, KayaValue arg);
+//void callFnID(void* vmptr, kint id, KayaValue arg);
 
 kint memusage();
 
@@ -200,7 +203,7 @@ void* do_opendir(wchar_t* name);
 kint do_closedir(void* dir);
 void* do_readdir(void* dir);
 
-wchar_t* dir_getname(void* ent);
+VMState* dir_getname(void* vmptr, void* ent);
 
 KayaValue do_stat(void* vm, wchar_t* name);
 
@@ -222,9 +225,10 @@ kint do_pclose(void* stream);
 kint nextPower(kint x);
 
 kint strAlloc(Value* str);
-Value* createString(kint len);
+Value* createString(void* vmptr, kint len);
 
 void do_GC_enable_incremental();
+void do_GC_finalizer(Value* obj, Value* finalizer);
 
 void do_sleep(kint len);
 void do_usleep(kint len);
