@@ -13,7 +13,7 @@ extern "C" {
     int peek(void* block,int offset);
 
     void pokestring(void* block, int offset, wchar_t* str);
-    wchar_t* peekstring(void* block, int offset);
+    wchar_t* peekstring(void* block, kint offset, kint bsize);
 
     void* copyBlock(void* block,int size);
     void* copyChunk(void* block,int offset,int size);
