@@ -118,8 +118,13 @@ int cipherBlockLength(int c)
 
 void* cipherOpen(KayaValue algo, KayaValue mode, int flags)
 {
+  int cflags = 0;
+  if (flags & 1) cflags += GCRY_CIPHER_SECURE;
+  if (flags & 2) cflags += GCRY_CIPHER_ENABLE_SYNC;
+  if (flags & 4) cflags += GCRY_CIPHER_CBC_CTS;
+  if (flags & 8) cflags += GCRY_CIPHER_CBC_MAC;
     GcryptData* d = new GcryptData();
-    gcry_cipher_open(&(d->handle), cipheralgo(algo), ciphermode(mode), flags);
+    gcry_cipher_open(&(d->handle), cipheralgo(algo), ciphermode(mode), cflags);
     d->algo = cipheralgo(algo);
     return (void*)d;
 }
@@ -166,7 +171,7 @@ void* do_encrypt(void* h, void* data, int size, KayaValue outsize)
     // is a multiple of blklen.
     newsize= size+blklen+(blklen-(size%blklen));
     newblock = KayaAlloc(newsize);
-    memcpy(newblock,data,newsize);
+    memcpy(newblock,data,size);
     data = newblock;
     size = newsize;
 
@@ -192,7 +197,9 @@ void* do_decrypt(void* h, void* data, int size, KayaValue outsize)
     // is a multiple of blklen.
     newsize= size+blklen+(blklen-(size%blklen));
     newblock = KayaAlloc(newsize);
-    memcpy(newblock,data,newsize);
+    //    cout<<"decrypt:"<<size<<":"<<newsize<<":"<<newblock<<":"<<data<<endl;
+    memcpy(newblock,data,size);
+    //    cout<<"copied"<<endl;
     data = newblock;
     size = newsize;
 
@@ -207,10 +214,15 @@ void* do_decrypt(void* h, void* data, int size, KayaValue outsize)
 
 void* hashOpen(KayaValue algo, int flags)
 {
+    int hflags = 0;
+    if (flags & 1) hflags += GCRY_MD_FLAG_SECURE;
+    if (flags & 2) hflags += GCRY_MD_FLAG_HMAC;
+
     GcryptHash* d = new GcryptHash();
-    gcry_md_open(&(d->handle), hashalgo(algo),flags);
+    gcry_md_open(&(d->handle), hashalgo(algo),hflags);
     d->size = gcry_md_get_algo_dlen(hashalgo(algo));
     d->algo = hashalgo(algo);
+
     return (void*)d;
 }
 
