@@ -17,6 +17,17 @@ public:
     wchar_t* after;
 };
 
+class CompiledRegex : public gc_cleanup
+{
+public:
+  CompiledRegex(pcre* r):regex(r) {}
+  ~CompiledRegex() {
+    free(regex);
+  }
+  pcre* regex;
+  
+};
+
 // ECB 26/2/07 - this should be done by throwing a kaya_internalError(),
 // catching the exception in the .k and rethrowing. This method should be 
 // reserved for built in exceptions, because it relies on a specific method
@@ -29,6 +40,7 @@ extern "C" {
 
 void* re_compile(void* vm, wchar_t* pattern, kint len, kint flagcode);
 void* re_match(void* vm, void* pc, wchar_t* str, kint len);
+kint re_quickmatch(void* vm, void* pc, wchar_t* str, kint len);
 
 Array* getmatches(void* mo);
 bool matched(void* mo);
