@@ -1,5 +1,7 @@
 #!/usr/bin/perl
 
+my $exitstatus = 0;
+
 sub runtest {
     my ($test, $update) = @_;
 
@@ -22,6 +24,7 @@ sub runtest {
 	print "success\n";
     } else {
 	if ($update == 0) {
+	    $exitstatus = 1;
 	    print "FAILURE\n";
 	} else {
 	    system("cp output expected");
@@ -85,3 +88,4 @@ foreach $test (@tests) {
     }
     
 }
+exit $exitstatus;
